local AddonName, SM = ...
--local libc = LibStub:GetLibrary("LibCompress")

local strsub = strsub
local strchar = strchar
local strfind = strfind
local strupper = strupper
local gsub = gsub

local codeData = {}
local count = 255

SM.CodeEncode = function( code )
	local data
	local length
	local pos, endPos, index, caps, AsIs, compString
	repeat
		data = codeData[count]
		caps = false
		AsIs = false
		
		if data.AsIs then
			AsIs = true
		elseif data.caps then
			data = codeData[count + 1]
			caps = true
		end
		for x = 1, #data do
			index = 1
			compString = data[x]
			if caps then
				compString = strupper(strsub(compString, 1, 1))..strsub(compString, 2)
			end
			
			repeat
				pos, endPos = strfind(code, compString, index)
				
				if pos and AsIs then
					index = pos + 1
					length = code:len()
					if ( pos == 1 or strfind( (strsub(code, pos-1,pos-1)), "%A") ) and
					( endPos == length or strfind( (strsub(code, endPos+1,endPos+1)), "%A") ) then
						if pos == 1 and endPos == length then
							code = strchar(count,x)
						elseif pos == 1 then
							code = strchar(count,x)..strsub(code, endPos + 1)
						elseif endPos == length then
							code = strsub(code, 1, pos - 1)..strchar(count,x)
						else
							code = strsub(code, 1, pos - 1)..strchar(count,x)..strsub(code, endPos + 1)
						end
					end
				elseif pos then
					code = gsub(code, compString, strchar(count,x))
				end
			until not(pos)
		end
		
		count = count - 1
	until not(codeData[count])
	--code = libc:Compress(code)

	return code
end

SM.CodeDecode = function( code )
	--code = libc:Decompress(code)
	local data
	local caps
	repeat
		data = codeData[count]
		caps = false
		if codeData[count].caps then
			data = codeData[count + 1]
			caps = true
		for x = 1, #data do
			if caps then
				code = gsub(code, strchar(count, x), (strupper(strsub(data[x], 1, 1))..strsub(data[x], 2)))
			else
				code = gsub(code, strchar(count, x), data[x])
			end
		end
		
		count = count - 1
	until not(codeData[count])
	return code
end


-- ****************************************************************************
-- ****************************************************************************
-- ** Changing tables once used will break the compression 					 **
-- ** Try to keep tables below or at 200 records (alteast below min table #) **
-- ****************************************************************************
-- ****************************************************************************

-- AsIs items will be an exact match  others will match even if part of a word
-- caps will match the table x + 1 with the first letter of each word capitalized

codeData[255] = {	-- Lua Code   209 commands
	"in ipairs",		"in pairs",				"bit.band", 			"bit.mod", 				"bit.rshift", 			"bit.arshift", 			"bit.bor", 			"bit.bnot", 		"bit.lshift", 			"bit.bxor",				-- [10]
	"coroutine.resume", "coroutine.yield",		"coroutine.status", 	"coroutine.wrap",		"coroutine.create", 	"coroutine.running",	"math.log", 		"math.acos", 		"math.ldexp", 			"math.pow",				-- [20]
	"math.tanh", 		"math.deg",				"math.tan", 			"math.cosh", 			"math.cos", 			"math.random", 			"math.sinh", 		"math.frexp", 		"math.ceil", 			"math.floor",			-- [30]
	"math.rad", 		"math.abs",				"math.sqrt", 			"math.modf", 			"math.asin", 			"math.min", 			"math.max", 		"math.fmod", 		"math.log10", 			"math.atan2",			-- [40]
	"math.exp", 		"math.sin",				"math.atan", 			"rtable.type", 			"rtable.pairs", 		"rtable.next", 			"rtable.insert", 	"rtable.ipairs", 	"rtable.maxn", 			"rtable.remove",		-- [50]
 	"rtable.sort", 		"rtable.unpack",		"rtable.rtgsub", 		"rtable.concat", 		"rtable.wipe", 			"rtable.newtable", 		"string.split", 	"string.match", 	"string.gmatch", 		"string.upper", 		-- [60]
	"string.gsub", 		"string.format",		"string.lower", 		"string.sub", 			"string.gfind", 		"string.rep", 			"string.char", 		"string.rtgsub", 	"string.join",			"string.reverse",		-- [70]
 	"string.byte", 		"string.trim",			"string.len", 			"string.find", 			"table.setn", 			"table.insert", 		"table.getn", 		"table.foreachi",	"table.maxn", 			"table.foreach",		-- [80]
 	"table.concat", 	"table.removemulti",	"table.sort", 			"table.wipe", 			"table.remove", 		"abs", 					"acos", 			"asin", 			"assert", 				"atan",					-- [90]
	"atan2", 			"bit",					"ceil", 				"collectgarbage", 		"cos", 					"coroutine", 			"date", 			"debugbreak", 		"debugdump", 			"debughook",			-- [100]
	"debuginfo", 		"debugload",			"debuglocals", 			"debugprint", 			"debugprofilestart",	"debugprofilestop", 	"debugstack", 		"debugtimestamp", 	"deg", 					"difftime",				-- [110]
	"error", 			"exp",					"floor", 				"forceinsecure", 		"foreach", 				"foreachi", 			"format", 			"frexp", 			"gcinfo", 				"geterrorhandler",		-- [120]
 	"getfenv",			"getglobal",			"getmetatable", 		"getn", 				"getprinthandler", 		"gmatch", 				"gsub", 			"hooksecurefunc", 	"ipairs", 				"issecure",				-- [130]
	"issecurevariable", "ldexp",				"loadstring", 			"log", 					"log10", 				"max", 					"message", 			"min", 				"mod", 					"math", 				-- [140]
	"newproxy", 		"next",					"pairs", 				"pcall", 				"print", 				"rad", 					"random", 			"rawequal", 		"rawget", 				"rawset",				-- [150]
	"rtable", 			"scrub",				"securecall",			"select", 				"seterrorhandler", 		"setfenv", 				"setglobal", 		"setmetatable", 	"setprinthandler", 		"sin",					-- [160]
	"sort", 			"sqrt",					"strbyte",				"strchar", 				"strconcat", 			"strfind", 				"strjoin", 			"strlen", 			"strlenutf8", 			"strlower",				-- [170]
	"strmatch", 		"strrep", 				"strrev",				"strsplit", 			"strsub", 				"strtrim", 				"strupper", 		"string", 			"tan", 					"time",					-- [180]
	"tinsert", 			"tonumber",				"tostring",				"tostringall", 			"tremove", 				"type", 				"table", 			"unpack", 			"wipe", 				"xpcall",				-- [190]
	"and", 				"arg",					"do",					"else", 				"elseif", 				"end", 					"for", 				"function", 		"if", 					"local",				-- [200]
	"nil", 				"not",					"or",					"repeat", 				"return", 				"then", 				"until", 			"while",			"sub",											-- [209]

	["AsIs"] = true,
}

codeData[254] = { -- Common WoW functions
	"BNSetCustomMessage",		"ClearRaidMarker",			"ConvertToRaid",			"C_MountJournal.GetMountInfo",	"C_MountJournal.GetNumMounts",	"C_MountJournal.Summon",	"C_MountJournal.Summon",	"C_PetJournal.GetPetInfoByIndex",	"C_PetJournal.SummonPetByGUID",	"DoReadyCheck", 			-- [10]
	"EjectPassengerFromSeat",	"ForceQuit",				"GetOptOutOfLoot",			"GetScript",					"GetXPExhaustion",				"Hide",						"HookScript",				"InCombatLockdown",					"InitiateRolePoll",				"IsFlyableArea",			-- [20]
	"IsInInstance",				"IsQuestFlaggedCompleted",	"LeaveParty",				"LFGTeleport",					"RestartGx",					"SetCVar",					"SetOptOutOfLoot",			"Show",								"ShowCloak",					"ShowHelm", 				-- [30] 
	"ShowingCloak",				"ShowingHelm",				"UnitVehicleSeatCount",		"UnitXPMax",					"ConvertToParty",				"UnitOnTaxi",				"UnitInVehicle",			"VehicleExit",						"IsMounted",					"TaxiRequestEarlyLanding", 	-- [40]
	"UnitLevel",				"Click",					"UnitXP",					"SendChatMsg", 																																																			-- [44]

	["AsIs"] = true,
}

codeData[253] = { -- Common WoW words
	"Battle-Net",						"EXECUTE_CHAT_LINE",			"MacroEditBox",			"nameplateShowEnemies",			"nameplateShowEnemyGuardians",		"nameplateShowEnemyPets",		"nameplateShowEnemyTotems",		"nameplateShowFriendlyGuardians",		"nameplateShowFriendlyPets",	"nameplateShowFriendlyTotems",	-- [10]
	"nameplateShowFriends",				"StaticPopup",					"toast",				"UnitNameEnemyGuardianName",	"UnitNameEnemyPetName",				"UnitNameEnemyPlayerName",		"UnitNameEnemyTotemName",		"UnitNameFriendlyGuardianName",			"UnitNameFriendlyPetName",		"UnitNameFriendlyPlayerName",	-- [20]
	"UnitNameFriendlySpecialNPCName",	"UnitNameFriendlyTotemName",	"UnitNameGuildTitle",	"UnitNameHostleNPC",			"UnitNameNonCombatCreatureName",	"UnitNameNPC",					"UnitNameOwn",					"UnitNamePlayerGuild",					"UnitNamePlayerPVPTitle",		"button",						-- [30]
	"LeftButton",																																																																															-- [31]

	["AsIs"] = true,
}

codeData[252] = { -- Common words in Slash Magic
	"attempt",			"boolean",				"cancel",				"cloak",				"combat",				"command",				"commands",			"counts",			"delayed",				"dismount",	-- [10]
	"down",				"experience",			"favorite",				"flight",				"flying",				"frames",				"from",				"greater",			"helm",					"hide",		-- [20]
	"initiate",			"land",					"logout",				"Logout",				"matching",				"mount",				"msg",				"number",			"once",					"optional",	-- [30]
	"party",			"passed",				"pet",					"pets",					"player",				"quit",					"quits",			"raid",				"saved",				"second",	-- [40]
	"select",			"show",					"slash",				"static",				"summons",				"summoning",			"taxi",				"taxis",			"toggle",				"unit",		-- [50]
	"vanity",			"visibility",			"whatever",				"cannot",				"summon",				"proper",				"usage",			"critter",			"showing",				"haven't",	-- [60]
	"completed",		"complete",				"uncomplete",			"passing",				"loot",					"delay",				"running",			"purchase",			"items",				"garrison",	-- [70]
	"resources",		"toast",				"cleared",				"longer",				"names",				"mins",					"secs",																		-- [77]

}

codeData[251] = { -- 1st letter caps for above
	["caps"] = true
}

-- 514 most commonly used words from a google search
codeData[250] = {	-- start of most common words and verbs   200 in this section
	"understand",	"translate",	"children",		"complain",		"complete",		"interest",		"language",		"mountain",		"organise",		"possible",	-- [10]
	"question",		"remember",		"sentence",		"thousand",		"together",		"turn off",		"against",		"be able",		"believe",		"between",	-- [20]
	"brought",		"certain",		"contain",		"correct",		"country",		"develop",		"example",		"explain",		"hundred",		"machine",	-- [30]
	"measure",		"morning",		"nothing",		"numeral",		"pattern",		"perhaps",		"picture",		"problem",		"produce",		"product",	-- [40]
	"science",		"several",		"special",		"succeed",		"surface",		"thought",		"through",		"turn on",		"wake up",		"accept",	-- [50]
	"always",		"animal",		"answer",		"appear",		"beauty",		"before",		"behind",		"better",		"borrow",		"cancel",	-- [60]
	"center",		"change",		"common",		"course",		"decide",		"differ",		"direct",		"during",		"enough",		"family",	-- [70]
	"father",		"figure",		"finish",		"follow",		"forget",		"friend",		"govern",		"ground",		"happen",		"island",	-- [80]
	"letter",		"listen",		"little",		"minute",		"mother",		"notice",		"number",		"object",		"people",		"person",	-- [90]
	"record",		"school",		"second",		"should",		"simple",		"street",		"strong",		"though",		"toward",		"travel",	-- [100]
	"weight",		"wonder",		"about",		"above",		"after",		"again",		"allow",		"among",		"began",		"begin",	-- [110]
	"begin",		"black",		"break",		"bring",		"build",		"carry",		"cause",		"check",		"class",		"clean",	-- [120]
	"clear",		"close",		"color",		"cough",		"could",		"count",		"cover",		"cross",		"dance",		"don't",	-- [130]
	"drink",		"drive",		"early",		"earth",		"every",		"field",		"final",		"first",		"force",		"found",	-- [140]
	"front",		"great",		"green",		"group",		"heard",		"horse",		"house",		"large",		"laugh",		"learn",	-- [150]
	"leave",		"light",		"might",		"money",		"music",		"never",		"night",		"north",		"often",		"order",	-- [160]
	"other",		"paper",		"piece",		"place",		"plain",		"plane",		"plant",		"point",		"pound",		"power",	-- [170]
	"press",		"quick",		"reach",		"ready",		"reply",		"right",		"river",		"round",		"serve",		"shape",	-- [180]
	"short",		"since",		"sleep",		"small",		"smoke",		"sound",		"south",		"speak",		"spell",		"spend",	-- [190]
	"stand",		"start",		"state",		"still",		"stood",		"story",		"study",		"table",		"teach",		"their",	-- [200]
}
codeData[249] = { -- 1st letter caps for above
	["caps"] = true
}

codeData[248] = {	-- most common continues   200 in this section
	"there",		"these",		"thing",		"think",		"those",		"three",		"under",		"until",		"usual",		"voice",	-- [10]
	"vowel",		"watch",		"water",		"wheel",		"where",		"which",		"while",		"white",		"whole",		"world",	-- [20]
	"worry",		"would",		"write",		"young",		"able",			"also",			"area",			"back",			"base",			"been",		-- [30]
	"best",			"bird",			"blue",			"boat",			"body",			"book",			"both",			"busy",			"call",			"came",		-- [40]
	"care",			"city",			"cold",			"comb",			"come",			"dark",			"deep",			"does",			"done",			"door",		-- [50]
	"down",			"draw",			"each",			"ease",			"east",			"even",			"ever",			"face",			"fact",			"fall",		-- [60]
	"farm",			"fast",			"feel",			"feet",			"fill",			"find",			"fine",			"fire",			"fish",			"five",		-- [70]
	"food",			"foot",			"form",			"four",			"free",			"from",			"full",			"game",			"gave",			"girl",		-- [80]
	"give",			"gold",			"good",			"grow",			"half",			"hand",			"hard",			"have",			"head",			"hear",		-- [90]
	"heat",			"help",			"here",			"high",			"hold",			"home",			"hour",			"hurt",			"idea",			"inch",		-- [100]
	"just",			"keep",			"kind",			"king",			"knew",			"know",			"land",			"last",			"late",			"lead",		-- [110]
	"left",			"less",			"life",			"like",			"line",			"list",			"live",			"long",			"look",			"lose",		-- [120]
	"love",			"made",			"main",			"make",			"many",			"mark",			"mean",			"mile",			"mind",			"miss",		-- [130]
	"moon",			"more",			"most",			"move",			"much",			"must",			"name",			"near",			"need",			"next",		-- [140]
	"note",			"noun",			"once",			"only",			"open",			"over",			"page",			"part",			"pass",			"plan",		-- [150]
	"play",			"port",			"pose",			"pull",			"rain",			"read",			"real",			"rest",			"road",			"rock",		-- [160]
	"room",			"rule",			"said",			"same",			"seem",			"self",			"sell",			"send",			"ship",			"show",		-- [170]
	"shut",			"side",			"sign",			"sing",			"size",			"slow",			"snow",			"some",			"song",			"soon",		-- [180]
	"star",			"stay",			"step",			"stop",			"such",			"sure",			"swim",			"tail",			"take",			"talk",		-- [190]
	"tell",			"test",			"than",			"that",			"them",			"then",			"they",			"this",			"time",			"told",		-- [200]

}
codeData[247] = { -- 1st letter caps for above
	["caps"] = true
}

codeData[246] = { -- most common continues	  114 in this section
	"took",			"town",			"tree",			"true",			"turn",			"type",			"unit",			"very",			"wait",			"walk",	-- [10]
	"want",			"warm",			"week",			"well",			"went",			"were",			"west",			"what",			"when",			"will",	-- [20]
	"wind",			"with",			"wood",			"word",			"work",			"year",			"your",			"act",			"add",			"age",	-- [30]
	"ago",			"air",			"all",			"and",			"any",			"are",			"ask",			"bed",			"big",			"box",	-- [40]
	"boy",			"but",			"buy",			"can",			"car",			"cry",			"cut",			"day",			"did",			"dog",	-- [50]
	"dry",			"eat",			"end",			"eye",			"far",			"few",			"fit",			"fix",			"fly",			"for",	-- [50]
	"get",			"got",			"had",			"has",			"her",			"him",			"his",			"hot",			"how",			"lay",	-- [70]
	"let",			"lot",			"low",			"man",			"map",			"may",			"men",			"new",			"now",			"off",	-- [80]
	"old",			"one",			"our",			"out",			"own",			"pay",			"put",			"ran",			"red",			"run",	-- [90]
	"saw",			"say",			"sea",			"see",			"set",			"she",			"sit",			"six",			"sun",			"ten",	-- [100]
	"the",			"too",			"top",			"try",			"two",			"use",			"war",			"was",			"way",			"who",	-- [110]
	"why",			"yes",			"yet",			"you",																									-- [114]

}
codeData[245] = { -- 1st letter caps for above
	["caps"] = true
}

codeData[244] = { -- most common quints 1 - 200
	"ofthe",		"ation",		"inthe",		"there",		"ingth",		"tothe",		"ngthe",		"other",		"atthe",		"tions",  	-- [10]
	"andth",		"ndthe",		"onthe",		"edthe",		"their",		"tiona",		"orthe",		"forth",		"ingto",		"theco",  	-- [20]
	"ction",		"which",		"these",		"after",		"eofth",		"about",		"erthe",		"ional",		"first",		"would",	-- [30]
	"witht",		"sthat",		"asthe",		"ition",		"inter",		"fromt",		"ithth",		"ththe",		"thefi",		"state",	-- [40]
	"ionof",		"esand",		"thatt",		"hatth",		"thing",		"tiono",		"count",		"nthes",		"ement",		"natio",	-- [50]
	"romth",		"tofth",		"thest",		"years",		"thepr",		"dthat",		"utthe",		"eople",		"peopl",		"omthe",	-- [60]
	"dwith",		"witha",		"thema",		"ought",		"hewas",		"softh",		"ember",		"rough",		"sinth",		"round",	-- [70]
	"swith",		"cause",		"ingin",		"under",		"ethat",		"eason",		"bythe",		"uring",		"ratio",		"themo",	-- [80]
	"enthe",		"erand",		"swere",		"ssion",		"again",		"edint",		"point",		"itwas",		"where",		"ident",	-- [90]
	"ingan",		"efore",		"schoo",		"every",		"chool",		"ments",		"esthe",		"could",		"onand",		"thene",	-- [100]
	"rethe",		"ewith",		"ecaus",		"edwit",		"stand",		"hough",		"thela",		"thoug",		"fthes",		"thats",	-- [110]
	"publi",		"compa",		"playe",		"dinth",		"hefir",		"ether",		"becau",		"nclud",		"nding",		"andin",	-- [120]
	"throu",		"three",		"efirs",		"ringt",		"isthe",		"tiont",		"inclu",		"ectio",		"thesa",		"durin",	-- [130]
	"hroug",		"secon",		"roduc",		"eside",		"einth",		"edfor",		"while",		"event",		"catio",		"start",	-- [140]
	"sfrom",		"place",		"econd",		"thein",		"theba",		"befor",		"right",		"thati",		"theca",		"herea",	-- [150]
	"forma",		"ntoth",		"seaso",		"nthec",		"offic",		"nment",		"world",		"ating",		"edand",		"thede",	-- [160]
	"tedto",		"sandt",		"contr",		"esaid",		"tthat",		"versi",		"thepa",		"ouldb",		"dingt",		"erson",	-- [170]
	"ative",		"outth",		"menta",		"tioni",		"gover",		"feren",		"being",		"ainst",		"latio",		"ather",	-- [180]
	"thers",		"ngand",		"thath",		"nning",		"oneof",		"terth",		"elect",		"erfor",		"uldbe",		"ditio",	-- [190]
	"tatio",		"etime",		"nothe",		"tingt",		"oweve",		"wever",		"overn",		"erica",		"nthef",		"howev",	-- [200]
}

codeData[243] = { -- most common quints continues 201 - 400
	"thepo",		"thera",		"ivers",		"resid",		"meric",		"siden",		"oreth",		"thatw",		"ities",		"ameri",	-- [10]
	"ublic",		"heres",		"shave",		"thewo",		"ected",		"speci",		"produ",		"verth",		"tsand",		"rsand",	-- [20]
	"ompan",		"membe",		"still",		"think",		"tedth",		"andre",		"enter",		"ually",		"edtha",		"those",	-- [30]
	"house",		"illbe",		"llion",		"gains",		"forme",		"hesta",		"chang",		"theri",		"fthec",		"aking",	-- [40]
	"thech",		"epres",		"entof",		"butth",		"ffere",		"thewa",		"hepro",		"nofth",		"shoul",		"hould",	-- [50]
	"theth",		"ision",		"nthep",		"early",		"illio",		"willb",		"compl",		"tting",		"famil",		"times",	-- [60]
	"owing",		"prese",		"ingof",		"twith",		"lling",		"nthem",		"rican",		"ionan",		"gener",		"pport",	-- [70]
	"found",		"overt",		"going",		"great",		"sbeen",		"thero",		"thefo",		"ertha",		"hecom",		"hange",	-- [80]
	"erest",		"havet",		"hewor",		"toget",		"themi",		"provi",		"press",		"edbyt",		"anoth",		"ering",	-- [90]
	"later",		"andco",		"enera",		"theun",		"sever",		"tedin",		"ently",		"tthes",		"layer",		"wasth",	-- [100]
	"tment",		"dbyth",		"nther",		"tical",		"servi",		"resen",		"ecomp",		"ingfo",		"since",		"ionin",	-- [110]	
	"fthem",		"edtot",		"tionw",		"nthat",		"salso",		"hasbe",		"ithin",		"etwee",		"north",		"hecou",	-- [120]	
	"tween",		"rther",		"progr",		"betwe",		"hesai",		"eport",		"evelo",		"havea",		"nthei",		"sands",	-- [130]	
	"never",		"esent",		"thebe",		"reate",		"dtoth",		"thena",		"neoft",		"estat",		"thata",		"theen",	-- [140]	
	"vethe",		"hecon",		"large",		"ature",		"child",		"south",		"somet",		"herei",		"efrom",		"llthe",	-- [150]	
	"light",		"story",		"umber",		"trans",		"nwith",		"rofth",		"colle",		"erati",		"group",		"intot",	-- [160]	
	"ngfor",		"ernme",		"dfrom",		"epart",		"perat",		"evera",		"thefa",		"rnmen",		"appea",		"suppo",	-- [170]	
	"henth",		"serve",		"nthet",		"tinth",		"emain",		"ctive",		"vernm",		"power",		"field",		"ution",	-- [180]	
	"ordin",		"parti",		"sonth",		"witho",		"retha",		"posit",		"cover",		"ethin",		"ionto",		"ethan",	-- [190]	
	"ebeen",		"allth",		"theto",		"onoft",		"sethe",		"final",		"month",		"ollow",		"dthes",		"ingon",	-- [200]	
}

codeData[242] = { -- quints continues 401 - 600
	"fthep",		"ining",		"haveb",		"thete",		"eyear",		"theme",		"force",		"ecent",		"struc",		"mbers",	-- [10]
	"ystem",		"consi",		"sthes",		"syste",		"numbe",		"velop",		"ished",		"nside",		"veral",		"aroun",	-- [20]
	"theso",		"eturn",		"ountr",		"vebee",		"creat",		"ccess",		"commu",		"ontin",		"ervic",		"follo",	-- [30]
	"direc",		"iness",		"ionsa",		"dents",		"histo",		"asbee",		"irect",		"devel",		"ntheb",		"perso",	-- [40]
	"theya",		"thegr",		"alled",		"rvice",		"night",		"iffer",		"thebo",		"ility",		"mmuni",		"tther",	-- [50]
	"niver",		"entin",		"artic",		"diffe",		"conti",		"thert",		"etter",		"ityof",		"esame",		"reall",	-- [60]
	"strat",		"ftert",		"hesam",		"tinga",		"perce",		"nsand",		"thetr",		"missi",		"marke",		"ommun",	-- [70]
	"bilit",		"thele",		"esign",		"rcent",		"recor",		"istor",		"theho",		"moret",		"ecame",		"avebe",	-- [80]
	"gwith",		"calle",		"essio",		"thesu",		"theyw",		"ecord",		"ffice",		"boutt",		"becom",		"ngwit",	-- [90]
	"ience",		"opera",		"outof",		"polic",		"hathe",		"ethes",		"repor",		"tthey",		"estha",		"ughth",	-- [100]	
	"order",		"tates",		"esare",		"ercen",		"hingt",		"ofhis",		"ogram",		"ionth",		"tedby",		"mento",	-- [110]
	"terna",		"unive",		"sting",		"ecomm",		"rogra",		"ights",		"clude",		"sions",		"icall",		"swill",	-- [120]
	"theti",		"ersan",		"ppear",		"aying",		"icati",		"eandt",		"thisi",		"inghi",		"heuni",		"nders",	-- [140]
	"milli",		"truct",		"neral",		"mbero",		"thedi",		"nthis",		"iving",		"erent",		"uppor",		"ablet",	-- [150]
	"infor",		"aving",		"thelo",		"uctio",		"ersit",		"sanda",		"spect",		"build",		"dthec",		"ption",	-- [160]
	"cethe",		"inand",		"presi",		"known",		"sucha",		"whent",		"ositi",		"froma",		"andhe",		"class",	-- [170]
	"rding",		"ntinu",		"bleto",		"etoth",		"ealth",		"lecti",		"andma",		"thega",		"dther",		"stthe",	-- [180]
	"rform",		"porta",		"lythe",		"retur",		"eplay",		"withh",		"yofth",		"sines",		"busin",		"socia",	-- [190]
	"egame",		"tyear",		"ninth",		"herew",		"andso",		"nthen",		"along",		"lease",		"usine",		"amily",	-- [200]
}

codeData[241] = { -- quads 0 - 200
	"tion",			"nthe",			"ther",			"that",			"ofth",			"fthe",			"thes",			"with",			"inth",			"atio",		-- [10]
	"othe",			"tthe",			"dthe",			"ingt",			"ethe",			"sand",			"sthe",			"here",			"thec",			"ment",		-- [20]
	"them",			"rthe",			"thep",			"from",			"this",			"ting",			"thei",			"ngth",			"ions",			"andt",		-- [30]
	"onth",			"toth",			"edto",			"thef",			"they",			"have",			"edth",			"inga",			"ndth",			"thet",		-- [40]
	"ould",			"orth",			"ever",			"then",			"ight",			"eand",			"atth",			"theb",			"over",			"ding",		-- [50]
	"iona",			"gthe",			"fort",			"thea",			"ring",			"ally",			"ands",			"efor",			"ings",			"edin",		-- [60]
	"anda",			"thin",			"ough",			"were",			"erth",			"year",			"will",			"thel",			"said",			"heir",		-- [70]
	"nter",			"ents",			"econ",			"ning",			"comp",			"tand",			"sion",			"time",			"ound",			"form",		-- [80]
	"nand",			"land",			"thew",			"king",			"port",			"ythe",			"hthe",			"heco",			"ated",			"andi",		-- [90]
	"thed",			"inte",			"hing",			"ngto",			"some",			"thee",			"also",			"enti",			"onal",			"ctio",		-- [100]
	"more",			"ance",			"ewas",			"stat",			"side",			"part",			"than",			"stha",			"eoft",			"hese",		-- [110]
	"sare",			"theo",			"rand",			"cent",			"edby",			"theg",			"play",			"ence",			"hich",			"sing",		-- [120]
	"whic",			"cont",			"even",			"afte",			"fter",			"when",			"mber",			"esto",			"ingi",			"ical",		-- [130]
	"nder",			"tive",			"pres",			"comm",			"ingo",			"abou",			"iono",			"stan",			"hewa",			"asth",		-- [140]
	"sfor",			"irst",			"into",			"ecom",			"bout",			"esta",			"dent",			"work",			"hatt",			"able",		-- [150]
	"ount",			"tobe",			"reat",			"inst",			"hest",			"ndin",			"dand",			"etha",			"ters",			"firs",		-- [160]
	"woul",			"been",			"thth",			"itht",			"esth",			"hesa",			"sint",			"esan",			"ture",			"vers",		-- [170]
	"very",			"onof",			"twas",			"reth",			"yand",			"ster",			"coun",			"theh",			"ling",			"soft",		-- [180]
	"fore",			"enta",			"stor",			"cons",			"tate",			"itio",			"athe",			"itha",			"enth",			"ears",		-- [190]
	"tedt",			"ates",			"hefi",			"eral",			"ater",			"erin",			"nati",			"eres",			"like",			"rati",		-- [200]
}

codeData[240] = { -- quads 201 - 400	
	"eint",			"ngin",			"utth",			"andw",			"romt",			"rest",			"esti",			"ught",			"ered",			"andc",		-- [10]
	"andh",			"thou",			"mthe",			"ason",			"ecti",			"emen",			"eran",			"call",			"ersa",			"what",		-- [20]
	"dint",			"ntin",			"wasa",			"late",			"erea",			"alle",			"dfor",			"cial",			"toft",			"unde",		-- [30]
	"ersi",			"ande",			"ento",			"illi",			"erat",			"erso",			"nfor",			"rent",			"esin",			"serv",		-- [40]
	"rate",			"stra",			"beca",			"chan",			"most",			"ando",			"ange",			"just",			"itis",			"game",		-- [50]
	"eare",			"ican",			"back",			"reco",			"omth",			"hema",			"ving",			"iden",			"iver",			"hepr",		-- [60]
	"vent",			"ress",			"real",			"area",			"oint",			"dtha",			"star",			"hand",			"each",			"ente",		-- [70]
	"ries",			"ople",			"erof",			"come",			"andp",			"ains",			"edan",			"ines",			"epro",			"isth",		-- [80]
	"edon",			"theu",			"fora",			"only",			"gain",			"arti",			"hers",			"rese",			"swit",			"ingf",		-- [90]
	"spec",			"andm",			"eopl",			"tter",			"peop",			"thre",			"dwit",			"know",			"erst",			"inal",		-- [100]
	"embe",			"inan",			"sinc",			"acti",			"stin",			"long",			"ause",			"caus",			"vera",			"esof",		-- [110]
	"eren",			"ffic",			"ingh",			"line",			"vert",			"thro",			"roug",			"ying",			"easo",			"andr",		-- [120]
	"iont",			"llow",			"urin",			"onan",			"swer",			"hist",			"roun",			"cess",			"byth",			"tain",		-- [130]
	"ethi",			"scho",			"lect",			"atin",			"last",			"onst",			"snot",			"your",			"reas",			"rica",		-- [140]
	"ngan",			"char",			"essi",			"high",			"tore",			"cond",			"many",			"ioni",			"tert",			"onto",		-- [150]
	"tfor",			"team",			"hemo",			"ctor",			"hats",			"take",			"epar",			"ders",			"lati",			"name",		-- [160]
	"orma",			"andf",			"seas",			"choo",			"unit",			"hene",			"outh",			"llin",			"tera",			"ffer",		-- [170]
	"hati",			"epla",			"noth",			"well",			"allo",			"ties",			"dont",			"tern",			"prov",			"ingw",		-- [180]
	"ract",			"seve",			"onsi",			"usin",			"ness",			"wher",			"chin",			"lled",			"itwa",			"make",		-- [190]
	"ingc",			"ende",			"ssio",			"agai",			"resi",			"poin",			"poli",			"sign",			"edwi",			"ower",		-- [200]
}

codeData[239] = { -- quads 401 - 600	
	"ntof",			"outt",			"cted",			"down",			"mand",			"efir",			"show",			"ewit",			"tati",			"epre",		-- [10]
	"stri",			"arge",			"hool",			"offi",			"crea",			"rfor",			"isto",			"mple",			"heca",			"ecau",		-- [20]
	"andb",			"test",			"atte",			"hela",			"fere",			"atur",			"laye",			"hera",			"turn",			"ntha",		-- [30]
	"sent",			"rtha",			"meth",			"lthe",			"appe",			"coul",			"seco",			"wast",			"used",			"ease",		-- [40]
	"ewor",			"ilit",			"oret",			"nclu",			"ouse",			"alth",			"ined",			"ubli",			"east",			"ious",		-- [50]
	"ompa",			"oper",			"erto",			"ontr",			"andd",			"hath",			"howe",			"incl",			"omet",			"eing",		-- [60]
	"evel",			"heha",			"stre",			"pers",			"heth",			"bein",			"ship",			"tory",			"houg",			"ince",		-- [70]
	"ntot",			"hewo",			"teda",			"hose",			"amer",			"este",			"main",			"edfo",			"iste",			"hear",		-- [80]
	"ecou",			"publ",			"tint",			"ingp",			"rman",			"estr",			"ingl",			"clud",			"lace",			"hede",		-- [90]
	"earl",			"indi",			"cati",			"nded",			"eric",			"ttle",			"hous",			"came",			"leas",			"onin",		-- [100]
	"ener",			"oneo",			"rson",			"read",			"hein",			"gand",			"hrou",			"ward",			"esse",			"teri",		-- [110]
	"lity",			"eate",			"hree",			"wing",			"plac",			"tart",			"esid",			"expe",			"forc",			"grou",		-- [120]
	"head",			"cord",			"owin",			"stof",			"heri",			"befo",			"isti",			"plan",			"hile",			"hatw",		-- [130]
	"duri",			"sout",			"ttha",			"ishe",			"whil",			"diti",			"esen",			"ames",			"andl",			"rece",		-- [140]
	"home",			"edas",			"less",			"hiss",			"nint",			"sfro",			"oduc",			"rodu",			"mean",			"city",		-- [150]
	"entt",			"ervi",			"oing",			"arte",			"neof",			"fina",			"istr",			"isco",			"arch",			"inin",		-- [160]
	"eall",			"esai",			"heba",			"worl",			"ming",			"asse",			"nmen",			"nted",			"ndre",			"weve",		-- [170]
	"inge",			"four",			"ingb",			"anin",			"onsa",			"hero",			"shed",			"erfo",			"tere",			"righ",		-- [180]
	"ille",			"hepa",			"nera",			"sist",			"lead",			"stru",			"hedi",			"ards",			"ased",			"ives",		-- [190]
	"ited",			"mark",			"arly",			"assi",			"meri",			"aren",			"orld",			"butt",			"inis",			"ours",		-- [200]
}

codeData[238] = { -- tris 1 - 200	
	"the",			"and",			"ing",			"ent",			"ion",			"her",			"for",			"tha",			"nth",			"int",		-- [10]
	"ere",			"tio",			"ter",			"est",			"ers",			"ati",			"hat",			"ate",			"all",			"eth",		-- [20]
	"hes",			"ver",			"his",			"oft",			"ith",			"fth",			"sth",			"oth",			"res",			"ont",		-- [30]
	"dth",			"are",			"rea",			"ear",			"was",			"sin",			"sto",			"tth",			"sta",			"thi",		-- [40]
	"tin",			"ted",			"ons",			"edt",			"wit",			"san",			"din",			"ort",			"con",			"rth",		-- [50]
	"eve",			"eco",			"era",			"ist",			"ngt",			"ast",			"ill",			"com",			"ore",			"ive",		-- [60]
	"nce",			"one",			"edi",			"pro",			"ess",			"out",			"ein",			"att",			"men",			"hec",		-- [70]
	"esa",			"hen",			"ina",			"eri",			"ert",			"ame",			"iti",			"ome",			"son",			"art",		-- [80]
	"man",			"ean",			"ona",			"eof",			"tor",			"hea",			"ran",			"rin",			"ine",			"eda",		-- [90]
	"nto",			"ave",			"nin",			"ove",			"oun",			"ain",			"ant",			"str",			"eto",			"hem",		-- [100]
	"sof",			"per",			"nde",			"ste",			"nte",			"eas",			"dto",			"our",			"red",			"rom",		-- [110]
	"tof",			"ght",			"tot",			"ese",			"cha",			"ica",			"hei",			"hin",			"ide",			"ndt",		-- [120]
	"han",			"tan",			"lin",			"not",			"der",			"ect",			"tra",			"igh",			"fro",			"eat",		-- [130]
	"sti",			"hep",			"ndi",			"ins",			"she",			"nal",			"pla",			"als",			"een",			"nti",		-- [140]
	"you",			"lan",			"und",			"nda",			"rat",			"lea",			"can",			"has",			"nds",			"nga",		-- [150]
	"hel",			"hed",			"inc",			"use",			"esi",			"gth",			"asa",			"het",			"nts",			"hav",		-- [160]
	"hew",			"tho",			"but",			"nan",			"ass",			"hef",			"ies",			"ret",			"end",			"par",		-- [170]
	"wer",			"cti",			"ren",			"rec",			"cal",			"its",			"ree",			"ene",			"rst",			"eal",		-- [180]
	"ana",			"nst",			"cou",			"tur",			"min",			"ity",			"yth",			"hey",			"eca",			"oul",		-- [190]
	"lle",			"ard",			"rou",			"anc",			"ost",			"pre",			"age",			"efo",			"les",			"ssi",		-- [200]
}

codeData[237] = { -- tris 201 - 400		
	"ema",			"eso",			"tat",			"ath",			"wor",			"ust",			"heb",			"ewa",			"sho",			"ind",		-- [10]
	"sed",			"hou",			"lly",			"uld",			"ase",			"ure",			"ono",			"ele",			"enc",			"nat",		-- [20]
	"ead",			"whe",			"ell",			"ble",			"kin",			"ans",			"tic",			"ali",			"sco",			"ero",		-- [30]
	"whi",			"ces",			"own",			"nta",			"act",			"ber",			"ven",			"tim",			"don",			"dan",		-- [40]
	"ose",			"ice",			"isa",			"ton",			"den",			"ngs",			"ugh",			"nes",			"lat",			"tal",		-- [50]
	"edo",			"ten",			"ime",			"eme",			"ack",			"tes",			"ple",			"ous",			"off",			"tto",		-- [60]
	"chi",			"ani",			"orm",			"ned",			"ens",			"sha",			"mor",			"iss",			"ite",			"nge",		-- [70]
	"tis",			"ora",			"lli",			"ede",			"sse",			"ade",			"rie",			"aid",			"emo",			"ral",		-- [80]
	"sit",			"oin",			"hth",			"tre",			"any",			"ake",			"ern",			"mer",			"ric",			"dis",		-- [90]
	"ish",			"oug",			"ini",			"ong",			"ntr",			"eli",			"wil",			"led",			"sar",			"how",		-- [100]
	"edb",			"ich",			"spe",			"sea",			"lit",			"yin",			"sai",			"ndo",			"gin",			"shi",		-- [110]
	"ord",			"mon",			"ena",			"new",			"por",			"ser",			"ial",			"ori",			"tte",			"mar",		-- [120]
	"epr",			"ach",			"har",			"yea",			"tri",			"che",			"tea",			"unt",			"omp",			"who",		-- [130]
	"tar",			"owe",			"rit",			"ded",			"ors",			"day",			"hee",			"thr",			"eir",			"ond",		-- [140]
	"mes",			"efi",			"had",			"ner",			"ela",			"let",			"lso",			"ris",			"ire",			"isi",		-- [150]
	"met",			"ars",			"hic",			"cen",			"ari",			"fin",			"tob",			"nsi",			"las",			"ope",		-- [160]
	"lar",			"des",			"fte",			"nit",			"sen",			"ang",			"som",			"abo",			"sio",			"two",		-- [170]
	"ian",			"eis",			"tsa",			"ngi",			"uni",			"ses",			"rep",			"rac",			"top",			"abl",		-- [180]
	"eti",			"ebe",			"eha",			"now",			"oni",			"ves",			"fir",			"erc",			"ofa",			"ace",		-- [190]
	"sal",			"get",			"app",			"ane",			"rsa",			"nof",			"heh",			"gre",			"win",			"car",		-- [200]
}	

codeData[236] = { -- tris 401 - 600		
	"ete",			"mat",			"cho",			"lay",			"swe",			"esp",			"pri",			"tiv",			"rof",			"gra",		-- [10]
	"llo",			"cor",			"eac",			"nis",			"dit",			"gan",			"gto",			"eno",			"bou",			"obe",		-- [20]
	"esh",			"tos",			"ery",			"rma",			"ngo",			"ewi",			"ara",			"rto",			"rel",			"oma",		-- [30]
	"ala",			"asi",			"tst",			"utt",			"irs",			"yan",			"lla",			"sfo",			"ork",			"ett",		-- [40]
	"lth",			"sid",			"aso",			"swi",			"ita",			"set",			"twa",			"erm",			"epa",			"ron",		-- [50]
	"tit",			"aft",			"dre",			"tle",			"mil",			"dby",			"ale",			"pen",			"bec",			"mbe",		-- [60]
	"toa",			"heg",			"sch",			"sis",			"rti",			"heo",			"low",			"lis",			"oll",			"war",		-- [70]
	"alt",			"elo",			"tro",			"cat",			"med",			"lic",			"hil",			"ile",			"tht",			"rem",		-- [80]
	"rre",			"ays",			"oli",			"rso",			"nsa",			"omm",			"old",			"cre",			"ata",			"ise",		-- [90]
	"cia",			"pos",			"ger",			"sma",			"uti",			"sts",			"sec",			"sbe",			"eni",			"sre",		-- [100]
	"lon",			"isc",			"nse",			"nor",			"bee",			"ano",			"nco",			"fer",			"itt",			"sno",		-- [110]
	"epo",			"eon",			"eds",			"eam",			"esc",			"fic",			"ech",			"way",			"ved",			"ike",		-- [120]
	"alo",			"yof",			"ash",			"ote",			"ook",			"eta",			"erf",			"onc",			"emi",			"eci",		-- [130]
	"ats",			"erv",			"rsi",			"sst",			"ili",			"eed",			"ary",			"sso",			"mth",			"vel",		-- [140]
	"dat",			"mea",			"esu",			"uri",			"pan",			"rch",			"uth",			"spo",			"wou",			"ffe",		-- [150]
	"sel",			"rei",			"rge",			"rse",			"tom",			"usi",			"ega",			"sas",			"ssa",			"ato",		-- [160]
	"erw",			"ood",			"ama",			"sat",			"ece",			"mpl",			"tso",			"gen",			"arr",			"dea",		-- [170]
	"sca",			"dof",			"ual",			"dbe",			"ewo",			"nso",			"rte",			"vin",			"adi",			"ndw",		-- [180]
	"ndh",			"edf",			"swh",			"see",			"toc",			"tch",			"ewh",			"eba",			"onl",			"tem",		-- [190]
	"dwi",			"err",			"lec",			"lac",			"eop",			"tel",			"ami",			"ehe",			"dfo",			"ien",		-- [200]
}

codeData[235] = { -- tris 601 - 800		
	"uch",			"ndc",			"ely",			"dst",			"ici",			"edw",			"aus",			"nfo",			"ntt",			"nne",		-- [10]
	"ewe",			"sur",			"exp",			"bet",			"ket",			"inf",			"etr",			"yto",			"rde",			"rce",		-- [20]
	"omt",			"evi",			"vet",			"pec",			"rai",			"arl",			"yst",			"sou",			"him",			"ref",		-- [30]
	"lik",			"ges",			"cto",			"urn",			"fou",			"lls",			"rne",			"wha",			"tow",			"ndr",		-- [40]
	"das",			"ssu",			"spr",			"opl",			"rad",			"esw",			"onf",			"col",			"omi",			"duc",		-- [50]
	"mos",			"ark",			"tai",			"ick",			"hos",			"ult",			"emb",			"ign",			"hoo",			"too",		-- [60]
	"nea",			"itw",			"ppe",			"ffi",			"ula",			"nam",			"mis",			"ced",			"los",			"gam",		-- [70]
	"nas",			"reg",			"lie",			"olo",			"twe",			"ann",			"tod",			"ail",			"ota",			"iso",		-- [80]
	"aye",			"tco",			"gro",			"cam",			"efe",			"onw",			"bea",			"ngl",			"wee",			"edu",		-- [90]
	"rew",			"aga",			"til",			"ode",			"ory",			"erb",			"bac",			"len",			"nly",			"imp",		-- [100]
	"aro",			"ehi",			"amp",			"mpa",			"spa",			"ivi",			"ict",			"stt",			"net",			"rta",		-- [110]
	"hre",			"erh",			"eet",			"row",			"sor",			"epe",			"tie",			"nch",			"tru",			"mal",		-- [120]
	"cte",			"ndm",			"atc",			"fac",			"twi",			"orc",			"dco",			"ndp",			"fre",			"ofs",		-- [130]
	"arg",			"ima",			"dec",			"eng",			"ria",			"owi",			"eit",			"pol",			"ked",			"inn",		-- [140]
	"bli",			"dow",			"etw",			"clu",			"hol",			"air",			"rri",			"rig",			"sla",			"rop",		-- [150]
	"ofi",			"toh",			"non",			"ool",			"isp",			"mak",			"rev",			"equ",			"lyt",			"jus",		-- [160]
	"lya",			"ole",			"cit",			"pea",			"vea",			"kno",			"oca",			"tak",			"acc",			"cer",		-- [170]
	"ada",			"sup",			"ngw",			"peo",			"dso",			"rdi",			"tou",			"cau",			"rov",			"tfo",		-- [180]
	"stu",			"pin",			"tly",			"ied",			"ues",			"tsi",			"awa",			"ler",			"wel",			"epl",		-- [190]
	"hor",			"erp",			"hro",			"ute",			"oss",			"tbe",			"tyo",			"tog",			"heu",			"byt",		-- [200]
}

codeData[234] = { -- tris 801 - 1000		
	"ngf",			"ngh",			"hal",			"ban",			"ism",			"rol",			"atu",			"sam",			"hom",			"dhi",		-- [10]
	"ild",			"gai",			"rds",			"edh",			"ino",			"ego",			"egi",			"llt",			"arc",			"ram",		-- [20]
	"oto",			"cle",			"dhe",			"aco",			"orl",			"cas",			"riv",			"ofc",			"orn",			"rna",		-- [30]
	"urs",			"avi",			"nar",			"ubl",			"tti",			"mem",			"amo",			"rme",			"qui",			"que",		-- [40]
	"nee",			"ndf",			"inh",			"ink",			"eho",			"ncl",			"ock",			"vic",			"rts",			"ets",		-- [50]
	"leg",			"cla",			"tse",			"eor",			"nci",			"sli",			"erg",			"did",			"rco",			"bas",		-- [60]
	"loc",			"ebu",			"hig",			"asp",			"ext",			"egr",			"dia",			"tab",			"bri",			"cei",		-- [70]
	"ras",			"dal",			"alc",			"run",			"ppo",			"ofh",			"cke",			"atw",			"cea",			"ott",		-- [80]
	"bal",			"nsh",			"mai",			"dar",			"asb",			"rfo",			"vis",			"owa",			"ude",			"rsh",		-- [90]
	"ifi",			"loo",			"hit",			"uar",			"cet",			"ovi",			"ndb",			"tol",			"uto",			"lem",		-- [100]
	"bro",			"vid",			"rio",			"uct",			"try",			"efr",			"sde",			"dha",			"bel",			"bil",		-- [110]
	"tas",			"suc",			"ebo",			"oro",			"sev",			"mit",			"yco",			"nwh",			"neo",			"rvi",		-- [120]
	"dev",			"eun",			"wan",			"sig",			"ths",			"asc",			"dsa",			"tac",			"emp",			"ank",		-- [130]
	"rke",			"nhe",			"nme",			"nre",			"ros",			"ngc",			"may",			"unc",			"nic",			"nni",		-- [140]
	"ura",			"ful",			"oco",			"osi",			"def",			"ngr",			"tec",			"say",			"nbe",			"yre",		-- [150]
	"ntl",			"dge",			"sac",			"sfr",			"tsh",			"isn",			"ldi",			"arm",			"cur",			"eag",		-- [160]
	"ndl",			"eld",			"smo",			"org",			"llb",			"del",			"poi",			"iat",			"urt",			"ker",		-- [170]
	"sol",			"tma",			"mus",			"bei",			"qua",			"lys",			"dwa",			"nma",			"hip",			"rod",		-- [180]
	"efa",			"inv",			"isl",			"pon",			"yon",			"pas",			"lia",			"reo",			"lif",			"tia",		-- [190]
	"cri",			"nsu",			"rly",			"lbe",			"oha",			"ndd",			"ham",			"ept",			"ebr",			"bes",		-- [200]
}