local AddonName, SM = ...
local FriendlyAddonName = "Slash Magic"
local _ -- try to avoid some taint blame
SlashMagicAddonTable = SM

--[[ **************************************************************************
     * Add On Starts Here                                                     *
	 **************************************************************************
]]
	
-- Addon Frame
SM.Frame = CreateFrame("Frame")
SM.Frame:RegisterEvent("ADDON_LOADED")

-- Boolean Conversions
local on = true
local off = false

-- Bill's Utils
local addOptionMt = BillsUtils.addOptionMt
local StatColor = BillsUtils.StatColor
local SJprint = BillsUtils.SJprint
local OptSaveTF = BillsUtils.OptSaveTF
local Wait = BillsUtils.Wait
local DoIt = BillsUtils.DoIt
local color = BillsUtils.Color -- Text Colors

BillsUtils.Locals[#BillsUtils.Locals +1] = function ()
	addOptionMt = BillsUtils.addOptionMt
	StatColor = BillsUtils.StatColor
	SJprint = BillsUtils.SJprint
	OptSaveTF = BillsUtils.OptSaveTF
	Wait = BillsUtils.Wait
	DoIt = BillsUtils.DoIt
	color = BillsUtils.Color
end
	
-- Addon Settings
local cmdKey = "SLASHMAGIC_%s_CMD"
local SMS
local CurrentDataVersion = 2
SM.AddonKey = "SlashMagic"

SlashMagic_Settings = {} -- General addon settings   SAVED
SlashMagic_Commands = {} -- Slash Command code   SAVED
SlashMagic_CmdVars = {}  -- Slash Magic Command Variables   SAVED
SlashMagic_CmdVarsPC = {}  -- Slash Magic Per Character Command Variables   SAVED Per Char

SlashMagicStaticValues = {}  -- Per session values for each command  NOT SAVED

SM.DefaultCmdsLoaded = false

SM.SlashCommandFrame = CreateFrame("FRAME")
do
	local CF = SM.SlashCommandFrame
	CF.EventHandler = {}

	CF:SetScript("OnEvent", function(self, event, ...)
		if self.EventHandler[event] then
		    local static, saved, savedPC, exist
			exist = false
			
			for index, value in pairs(self.EventHandler[event]) do
				static = SlashMagicStaticValues[index]
				saved = SlashMagic_CmdVars[index]
				savedPC = SlashMagic_CmdVarsPC[index]
				value(self, event, ...)
				exist = true
			end
			if not(exist) then
				self.EventHandler[event] = nil
			end
		else
			self:UnregisterEvent(event)
		end
	end)
	
	SlashMagic_OnEvent = function(command, event, doWhat)
		if not( command ) or not( event ) then return end
		if type(doWhat) == "nil" then
			if CF.EventHandler[event] and CF.EventHandler[event][command] then
				CF.EventHandler[event][command] = nil
			end
		elseif type(doWhat) == "function" then
			CF:RegisterEvent( event )
			if not(CF.EventHandler[event]) then CF.EventHandler[event] = {} end
			CF.EventHandler[event][command] = doWhat
		end
	end
	
	local timeWaitTable = {}
	local e = 0
	CF:SetScript("OnUpdate", function(self, elapsed)
		e = e + elapsed
		if e < 0.1 then return end
		if timeWaitTable[1] then
			local static, saved, savedPC, command, seconds, again, funct, stop
			for i = #timeWaitTable, 1, -1 do
				seconds = timeWaitTable[i][2]
				seconds = seconds - e
				if seconds <= 0 then
					command = timeWaitTable[i][1]
					static = SlashMagicStaticValues[command]
					saved = SlashMagic_CmdVars[command]
					savedPC = SlashMagic_CmdVarsPC [command]
					again = timeWaitTable[i][3]
					funct = timeWaitTable[i][4]
					stop = funct()
					if again and not(stop) then
						table.insert(timeWaitTable, {command, again, again, funct })
					end
					table.remove(timeWaitTable, i)
				else 
					timeWaitTable[i][2] = seconds
				end
			end
		end
		e = 0
	end)

	SlashMagic_InTime = function(command, seconds, again, doWhat)
		if again and type(again) == "boolean" then again = seconds end
		table.insert(timeWaitTable, {command, seconds, again, doWhat} )
	end
end

SM.Settings_Defaults = {
	Enable = on,
	EnableAutoRuns = true,
	LoadDefaultCommands = true,
	EnableArchives = true,
	ShowGems = true,
	ShowDead = true,
	}

SM.VersionUpdate = function()
	print("Slash Magic is upgrading its data structures. It will be a second")
	local commands, command, tempCommand
	local temp = {}
	commands = SlashMagic_Commands
	for index, value in pairs(commands.code) do
		temp[index] = {}
		tempCommand = temp[index]
		tempCommand.desc = commands.desc[index]
		tempCommand.vers = commands.vers[index]
		tempCommand.alias = commands.alias[index]
		tempCommand.state = false
		tempCommand.code = {unpack(commands.code[index])}
	end
	table.wipe(commands)
	for index, value in pairs(temp) do
		commands[index] = {}
		command = commands[index]
		command.desc = value.desc
		command.vers = value.vers
		command.alias = value.alias
		command.state = value.state
		command.code = {unpack(value.code)}
	end
	SlashMagic_Settings.DataVer = CurrentDataVersion
	
	StaticPopupDialogs["SLASHMAGICDATAUPGRADE"] = {
	text = "Slash Magic has upgraded its data structure and as a result all existing(old) commands have been set off. Please open config to reset their state.",
	button1 = "Open Config",
	button2 = "Cancel",
	OnAccept = function()
		local name, title, notes, enabled, loadable, reason, security = GetAddOnInfo("SlashMagicConfig")
		local isLoaded = IsAddOnLoaded("SlashMagicConfig")
		if reason == "DISABLED" then
			EnableAddOn("SlashMagicConfig")
			LoadAddOn("SlashMagicConfig")
		elseif reason == "MISSING" then
			print("Slash Magic: The config gui is missing from the AddOns folder. Please reinstall SlashMagic to enable the gui.")
			return
		elseif loadable and not(isLoaded) then
			LoadAddOn("SlashMagicConfig")
		end
		
		if SM.ConfigFrame then
			SM.ConfigFrame:Show()
		else
			print("Slash Magic: There was an unknown error trying to open the config frame. Please enable the addon manually.")
		end
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,  -- avoid some UI taint, see http://www.wowace.com/announcements/how-to-avoid-some-ui-taint/ 
	}
	
	StaticPopup_Show("SLASHMAGICDATAUPGRADE")
end

SM.Enable = function( index )
	if not( SMS.Enable ) then
		return false
	end
	
	local playerName = UnitName("player")
	local realmName = GetRealmName()
	
	if not(SlashMagic_CmdVars[index]) then SlashMagic_CmdVars[index] = {} end
	if not(SlashMagic_CmdVarsPC[index]) then SlashMagic_CmdVarsPC[index] = {} end
	if not(SlashMagic_CmdVarsPC[index][realmName]) then SlashMagic_CmdVarsPC[index][realmName] = {} end
	if not(SlashMagic_CmdVarsPC[index][realmName][playerName]) then SlashMagic_CmdVarsPC[index][realmName][playerName] = {} end
	SlashMagicStaticValues[index] = {}
	
	local command = SlashMagic_Commands[index]
	
	if command and command.state and not(command.isDead) then
		local cmdName = cmdKey:format(strupper(index))
		local CodeCleanup = {unpack(command.code)}
		local found, xPos, yPos, tempString
		for x = 1, #CodeCleanup do
			CodeCleanup[x] = string.trim( CodeCleanup[x]) -- removes excess spaces at start / end

			-- Detect nestled quotes for comment detection so we don't get -- from part of a string
			-- and stray quotes that may interfere with quote pair detection
			-- only needs to run if a comment may be present ( -- is found )	
			if string.find(CodeCleanup[x], "%-%-") then
				tempString = CodeCleanup[x]
				tempString = string.gsub(tempString,"\\\"","xx") --remove escaped quotes
				tempString = string.gsub(tempString,"\\\'","xx") --remove escaped single quotes
				
				repeat --grabs innermost pair of quotes, replaces them, and then repeats until no quotes
					found = false
					xPos, yPos = string.find(tempString, "%-%-")
				
					if xPos then
						xPos,yPos = string.find(tempString, "\"[^\"]*\"")
						if xPos then
							tempString = string.sub(tempString,1,xPos-1)..string.rep("x", (yPos - xPos)+1)..string.sub(tempString, yPos+1 )
							found = true
						end
						xPos,yPos = string.find(tempString, "\'[^\']*\'")
						if xPos then
							tempString = string.sub(tempString,1,xPos-1)..string.rep("x", (yPos - xPos)+1)..string.sub(tempString, yPos)
							found = true
						end
					end
				until not(found)
				xPos, yPos = string.find(tempString, "%-%-")
				if xPos and xPos == 1 then
					CodeCleanup[x] = ""
				elseif xPos then
					CodeCleanup[x] = string.sub(CodeCleanup[x], 1, xPos - 1)
				end
			end
		end
		
		local headerCode = { "local _",
						    ("local saved = SlashMagic_CmdVars['%s']"):format(index),
							("local savedPC = SlashMagic_CmdVarsPC[\"%s\"][\"%s\"][\"%s\"]"):format(index, realmName, playerName),
						    ("local static = SlashMagicStaticValues['%s']"):format(index),
							 "local color = BillsUtils.Color",
						   }
						
		local header = string.join(string.char(13), unpack(headerCode))
		local code = string.join(string.char(13), header, unpack(CodeCleanup) )
		
		-- add in event/time functions if used
		code = string.gsub(code, "OnEvent%(", ("SlashMagic_OnEvent( \"%s\", "):format(index))
		code = string.gsub(code, "InTime%(", ("SlashMagic_InTime( \"%s\", "):format(index))
		
		-- add in delayed function handler
		code = string.gsub(code, "DoIt%(", "BillsUtils.DoIt%(")

		-- using loadstring instead of pcall+RunScript to catch errors loading as pcall doesn't work with RunScript
		-- with loadstring need to return a function in a function or we get to pass no paramaters to new function

		local newFunction, errorMsg = loadstring( ("return function(msg) %s end"):format(code) ) 	--catches errors without triggering wow's built in error handler
		if not(newFunction) then
			local _, pos = string.find(errorMsg, "]:")                             -- find end of code quote
			errorMsg = string.sub(errorMsg, pos+1)                                 -- remove code quote (as it doesn't even go past the header
			local lineNum = string.match(errorMsg, "^%d*")                         -- get line number of error
			errorMsg = string.gsub(errorMsg, lineNum, tonumber(lineNum) - #header) -- remove header length so line number is in user code
			StaticPopup_Show("SLASHMAGIC_FAILED_LOADING_COMMAND", index, errorMsg) -- show error message
			SlashMagic_Commands[index].state = false                               -- set code not to run so error doesn't happen again
			return false
		end
		
		SlashCmdList[cmdName] = newFunction() -- extract wanted function from wrapper function
		_G[("SLASH_%s1"):format(cmdName)] = ("/%s"):format( strlower(index) )

		if type(command.alias) == "string" and string.len(command.alias) > 0 then
			local aliases = { string.split(" ", command.alias) }
			for x = 1, #aliases do
				_G[("SLASH_%s%d"):format(cmdName, x + 1)] = ("/%s"):format( strlower(aliases[x]) )
			end
		end
		
		return true
	end
	return false
end

SM.Disable = function( index, force )
	if SlashMagic_Commands[index].state == false or force then
		local cmdName = cmdKey:format(strupper(index))
		local i=1;
		while _G["SLASH_"..cmdName..i]~=nil do
			local slash = strupper(_G["SLASH_"..cmdName..i]);
			_G["SLASH_"..cmdName..i]=nil;--   Removes SLASH_*
			hash_SlashCmdList[slash]=nil;-- Removes from hash_SlashCmdList
			i=i+1;
		end

		SlashCmdList[cmdName]=nil;--  Removes from SlashCmdList
		getmetatable(SlashCmdList).__index[cmdName]=nil;--    Removes from metatable
	end
end
	
SM.Reset1 = function( index ) -- Reset a single command to default
	if SM.Commands_Defaults[index] then
		SM.Disable( index, true )		
		table.wipe(SlashMagic_CmdVars[index])
		table.wipe(SlashMagic_CmdVarsPC[index])
		table.wipe(SlashMagicStaticValues[index])
		table.wipe(SlashMagic_Commands[index]) 
		addOptionMt(SlashMagic_Commands[index], SM.Commands_Defaults[index])
		SM.Enable( index )
		return true
	end
	return false
end

SM.Reset = function() -- Reset all commands to default and deletes custom commands
	for index, value in pairs( SlashMagic_Commands ) do
		if value then
			SM.Disable( index, true )
		end
	end
	
	table.wipe(SlashMagic_Settings)
	table.wipe(SlashMagic_CmdVars)
	table.wipe(SlashMagic_CmdVarsPC)
	table.wipe(SlashMagicStaticValues)
	table.wipe(SlashMagic_Commands)

	for index, value in pairs(SM.Commands_Defaults) do
		if not(SlashMagic_Commands[index]) then
			SlashMagic_Commands[index] = {}
		end
		addOptionMt(SlashMagic_Commands[index], value)
	end

	for index, value in pairs( SlashMagic_Commands ) do
		if value.state then
			SM.Enable( index )
		end
	end
end


SM.AddonEnable = function()
	local commands = SlashMagic_Commands
	for index, value in pairs(commands) do
		if value then
			SM.Enable( index )
		end
	end
end

SM.AddonDisable = function()
	local commands = SlashMagic_Commands
	for index, value in pairs(commands) do
		if value then
			SM.Disable( index, true )
		end
	end
end

SM.LoadManager = function(AddOn)
	local isLoaded = IsAddOnLoaded( AddOn )
	if not( isLoaded ) then
		local name, title, notes, enabled, loadable, reason, security = GetAddOnInfo( AddOn )
		if reason == "DISABLED" then
			EnableAddOn( AddOn )
			LoadAddOn( AddOn )
		elseif reason == "MISSING" then
			print("Slash Magic: "..AddOn.." is missing from the AddOns folder. Please reinstall SlashMagic to fix the missing files")
		elseif loadable and not(isLoaded) then
			LoadAddOn( AddOn )
		end	
	end
	return IsAddOnLoaded( AddOn )
end

-- ****************************************************************************
-- * Event Handler

SM.Frame:SetScript("OnEvent",function(self, event, ...)
	local arg = ...
	if event == "ADDON_LOADED" then
		if arg == AddonName then
			addOptionMt( SlashMagic_Settings , SM.Settings_Defaults )
			SMS = SlashMagic_Settings
		
			if not(SMS.CurrentDataVersion) or SMS.CurrentDataVersion < CurrentDataVersion then
				SMS.CurrentDataVersion = CurrentDataVersion
				--SM.VersionUpdate()
			end
		
			if SMS.LoadDefaultCommands then
				SM.DefaultCmdsLoaded = SM.LoadManager("SlashMagicDefaultCmds")
			else
				for index, value in pairs(SlashMagic_Commands) do
					if not(value.code) and not(value.archived) then
						SlashMagic_Commands[index] = nil
					end
				end
			end
	
			if type( SMS.AutoRuns ) ~= "table" then
				SMS.AutoRuns = {}
			end
		
			if SMS.Enable then
				for index, value in pairs( SlashMagic_Commands ) do
					if value.state then
						SM.Enable( index )
					end
				end
			end
			
			SJprint(color.Green, FriendlyAddonName, color.White, " ver:", GetAddOnMetadata( AddonName, "Version"), " loaded. type: \"/sm config\" to setup the addon", color.Reset)
			
			if SMS.EnableAutoRuns and ( #SMS.AutoRuns > 0 ) then
				SlashMagic_AutoRun_In = 15
				SM.Frame:SetScript("OnUpdate", function(self, elapsed)
					if SlashMagic_AutoRun_In then
						SlashMagic_AutoRun_In = SlashMagic_AutoRun_In - elapsed
						if SlashMagic_AutoRun_In < 0 then
							SJprint(color.Green, FriendlyAddonName, color.White, " auto-run commands are now running", color.Reset)
							for x = 1, #SMS.AutoRuns do
								if SMS.AutoRuns[x] ~= "" then
									Wait( (x - 1) * 5, DoIt, SMS.AutoRuns[x])
								end
							end
							SlashMagic_AutoRun_In = nil
							SM.Frame:SetScript("OnUpdate",nil)
						end
					end
				end)
			end
			
			SM.Frame:UnregisterEvent("ADDON_LOADED")
		else
			return
		end
		
		
--	elseif event == "PLAYER_LOGIN" then
-- 		if SMS.EnableRecieveCommands then
-- 			SM.Frame:RegisterEvent("CHAT_MSG_ADDON")
-- 			local didItWork = RegisterAddonMessagePrefix(SM.AddonKey)
-- 			if not(didItWork) then 
-- 				print("Slash Magic: There was a problem registering the addon to receive transmitted commands")
-- 			end
-- 		end

--		Wait( 15, function()
--			local tempCode, tempCode2
--			for key, value in pairs(SlashMagic_Commands) do
--				tempCode = string.join("\r", unpack(value.code))
--				tempCode2 = SM.CodeEncode(tempCode)
--				print(key, tempCode2:len() / tempCode:len(), tempCode2:len(), tempCode:len())
--				value.arc = tempCode2
--			end
--		end)
		
		
		
--	elseif event == "CHAT_MSG_ADDON" then
--		local prefix, message, dist, sender = ...
--		if prefix ~= SM.AddonKey then
--			return
--		end
--	end
--	return self, event, ...

	end
end)


-- ****************************************************************************
-- * Slash Command Handler Section
local SlashHandler = {}

SLASH_SLASHMAGIC1 = "/slashmagic"
SLASH_SLASHMAGIC2 = "/sm"

SlashCmdList["SLASHMAGIC"] = function(msg)
	msg = msg:lower()
	local cmd,arg,arg2 = string.split(" ", msg, 3)
	if not( arg ) then
		arg = ""
	end
	
	if SlashHandler[cmd] then -- handle major commands here
		SlashHandler[cmd](arg)
		return
	end
	
	if not(SMS.Enable) then -- if addon isn't enabled then you can't change command settings
		print(" ")
		SJprint( color.Yellow, FriendlyAddonName, color.Reset)
		SJprint( color.Yellow, "  This addon is ", color.Red, "DISABLED", color.Reset)
		SJprint( color.Yellow, "  Type: ", color.Blue, "/sm enable on", color.Reset)
		SJprint( color.Yellow, "  to enable.", color.Reset)
		return
	end
	
	-- check to see if passed command is a valid command or an alias to a valid command
	-- and if an alias change passed alias to command name
	if not(SlashMagic_Commands[cmd]) then
		local aliases
		local keySet = false
		for key, value in pairs(SlashMagic_Commands) do
			if value.alias then
				aliases = {string.split(" ", value.alias)}
				for x = 1, #aliases do
					if cmd == aliases[x] then
						cmd = key
						keySet = true
						break
					end
				end
				if keySet then
					break
				end
			end
		end
	end
	
	-- see if passed command is an actual command and if so enable/disable/reset
	if SlashMagic_Commands[cmd] then
		local command = SlashMagic_Commands[cmd]
		local cmdName = cmdKey:format( strupper(cmd) )
		if arg then
			local tobool = { 
				["on"] = true,    ["true"] = true,   ["1"] = true,  ["yes"] = true,  ["enable"] = true,
				["off"] = false,  ["false"] = false, ["0"] = false, ["no"] = false,  ["disable"] = false
			}
			
			if tobool[arg] then
				command.state = tobool[ arg ]
				if command.state then
					SM.Enable( cmd )
				else
					SM.Disable( cmd )
				end
				
			elseif arg == "reset" then
				if SM.Reset1( cmd ) then
					SJprint( color.Yellow, "Slash Magic:", color.Blue, cmd, color.Yellow, "has been reset to default.", color.Reset)
				else
					SJprint( color.Yellow, "Slash Magic:", color.Blue, cmd, color.Yellow, "is not a default command so it can not be reset.", color.Reset)
				end
			else
				SJprint(color.Yellow, "Slash Magic:", color.White, "valid options for commands are [on/off/reset] not", color.Red, arg, color.Reset)
			end
			return
		end
		
		SJprint( color.Yellow, "Slash Magic:", color.Blue, cmd, color.Yellow, "is set", (command.state and (color.Green.."ON") or (color.Red.."OFF")), color.Reset)
		return
	end
	
	print(" ")
	SJprint( color.Yellow, FriendlyAddonName, color.Reset)
	local slashCommand = "/"..strupper(cmd)
	if _G["hash_SlashCmdList"][slashCommand] or _G["hash_ChatTypeInfoList"][slashCommand] then
		SJprint( color.Yellow, "  Error: ", color.Red, cmd, color.Yellow, " is a valid slash command but it is not provided by this addon so we cannot control it here.", color.Reset)
	elseif _G["hash_EmoteTokenList"][slashCommand] then
		SJprint( color.Yellow, "  Error: ", color.Red, cmd, color.Yellow, " is an emote not provided by this addon so we cannot control it here", color.Reset)
	else
		SJprint( color.Yellow, "  Error: ", color.Red, cmd, color.Yellow, " is not a valid command nor command alias", color.Reset)
	end
	
	SJprint( color.Yellow, "  Type: ", color.Blue, "/sm ", color.Yellow, "for the addon status and command list", color.Reset)
	SJprint( color.Yellow, "  Type: ", color.Blue, "/sm ", "cmdlist", color.Yellow, "for command list help", color.Reset)
	return

end

SlashHandler.config = function()
	local isLoaded = SM.LoadManager("SlashMagicConfig")
	
	if SM.ConfigFrame then
		SM.ConfigFrame:Show()
	else
		print("Slash Magic: There was an unknown error trying to open the config frame. Please enable the addon manually.")
		return false
	end
	return true
end
SlashHandler.cfg = SlashHandler.config

SlashHandler.autoruns = function()
	if SlashHandler.config() then
		SM.ConfigFrame.AutoRunsMode:Click("LeftButton")
	end
end

SlashHandler.enable = function(arg)
	OptSaveTF( SMS, "Enable", arg)
	if SMS.Enable then
		SM.AddonEnable()
	else
		SM.AddonDisable()
	end
end

SlashHandler.reset = SM.Reset

SlashHandler.cmdlist = function() 
	local CmdList = {}
	for index in pairs(SlashMagic_Commands) do
		CmdList[#CmdList +1] = index
	end
	table.sort(CmdList)
	SJprint(color.Yellow, FriendlyAddonName, color.Reset)
	SJprint(color.Yellow, "This is a list of all available commands and whether they are enabled or not", color.Reset)
	print(" ")
	
	local command
	local message = "%s%d. %s/%s%s     %s"
	local index = 0
	for x = 1, #CmdList do
		command = SlashMagic_Commands[CmdList[x]]
		if not(command.isDead) then
			index = index + 1
			print( message:format(color.White, index, ( command.state and color.Green or color.Red ), CmdList[x], color.Reset, (command.desc and command.desc or "No Description" ) ))
			print(" ")
		end
	end
	SJprint(color.Yellow, "Active commands are listed as", color.Green, "GREEN", color.Yellow, "while inactive commands are", color.Red, "RED", color.Reset)
	SJprint(color.Yellow, "Type:", color.Blue, "/sm COMMANDNAME on/off", color.Yellow, "to change a commands state", color.Reset)
	SJprint(color.Yellow, "Type:", color.Blue, "/sm config", color.Yellow, "to edit commands and settings", color.Reset)
end
SlashHandler.help = SlashHandler.cmdlist
SlashHandler[""] = SlashHandler.cmdlist

-- Popup if custom command fails to load
StaticPopupDialogs["SLASHMAGIC_FAILED_LOADING_COMMAND"] = {
	text = "Slash Magic\n\nError loading command: %s\n\n Error message returned:\n\n%s",
	button1 = OKAY,
	button2 = nil,
	OnAccept = nil,
	timeout = 0,
	whileDead = true,
	hideOnEscape = false,
	hasEditBox = false,
	showAlert  = true,
	OnShow = nil,
	preferredIndex = 3,  -- avoid some UI taint, see http://www.wowace.com/announcements/how-to-avoid-some-ui-taint/ 
}
