local AddonName, AddonTable = ...
local SM = SlashMagicAddonTable

-- Bill's Utils
local ChkBox = BillsUtils.ChkBox
local CheckClick = BillsUtils.CheckClick
local SliderMW = BillsUtils.SliderMW
local Wait = BillsUtils.Wait
local color = BillsUtils.Color -- Text Colors
local DoIt = BillsUtils.DoIt

BillsUtils.Locals[#BillsUtils.Locals +1] = function ()
	ChkBox = BillsUtils.ChkBox
	CheckClick = BillsUtils.CheckClick
	SliderMW = BillsUtils.SliderMW
	Wait = BillsUtils.Wait
	color = BillsUtils.color
	DoIt = BillsUtils.DoIt
end

-- ****************************************************************************
-- * Color Changing Button Stuff                                              *
-- ****************************************************************************

local ButtonTextures = {
	RED = {
		up = "Interface\\Buttons\\UI-Panel-Button-Up",
		down = "Interface\\Buttons\\UI-Panel-Button-Down",
		highlight = "Interface\\Buttons\\UI-Panel-Button-Highlight",
	},
	GREEN = {
		up = "Interface\\AddOns\\SlashMagicConfig\\ButtonSkins\\Green-Panel-Button-Up",
		down = "Interface\\AddOns\\SlashMagicConfig\\ButtonSkins\\Green-Panel-Button-Down",
		highlight = "Interface\\AddOns\\SlashMagicConfig\\ButtonSkins\\Green-Panel-Button-Highlight",
	},
	BLUE = {
		up = "Interface\\AddOns\\SlashMagicConfig\\ButtonSkins\\Blue-Panel-Button-Up",
		down = "Interface\\AddOns\\SlashMagicConfig\\ButtonSkins\\Blue-Panel-Button-Down",
		highlight = "Interface\\AddOns\\SlashMagicConfig\\ButtonSkins\\Blue-Panel-Button-Highlight",
	},
	GREY = {
		up = "Interface\\AddOns\\SlashMagicConfig\\ButtonSkins\\Grey-Panel-Button-Up",
		down = "Interface\\AddOns\\SlashMagicConfig\\ButtonSkins\\Grey-Panel-Button-Down",
		highlight = "Interface\\AddOns\\SlashMagicConfig\\ButtonSkins\\Grey-Panel-Button-Highlight",
	},
	BLACK = {
		up = "Interface\\AddOns\\SlashMagicConfig\\ButtonSkins\\Black-Panel-Button-Up",
		down = "Interface\\AddOns\\SlashMagicConfig\\ButtonSkins\\Black-Panel-Button-Down",
		highlight = "Interface\\AddOns\\SlashMagicConfig\\ButtonSkins\\Black-Panel-Button-Highlight",
	},
}

local function GetSkin(self)
	return self.__skin
end

local function SetSkin(self, skin)
	if self.__skin == skin then return end
	self.__skin = skin

	if self:IsMouseOver() and IsMouseButtonDown() then
		local down = ButtonTextures[skin].down
		self.Left:SetTexture(down)
		self.Right:SetTexture(down)
		self.Middle:SetTexture(down)
	else
		local up = ButtonTextures[skin].up
		self.Left:SetTexture(up)
		self.Right:SetTexture(up)
		self.Middle:SetTexture(up)
	end

	self:SetHighlightTexture(ButtonTextures[skin].highlight, "ADD")
	self:GetHighlightTexture():SetTexCoord(0, 80/128, 0, 22/32)
end

local function OnMouseDown(self, button)
	local down = ButtonTextures[self.__skin].down
	self.Left:SetTexture(down)
	self.Right:SetTexture(down)
	self.Middle:SetTexture(down)
end

local function OnMouseUp(self, button)
	local up = ButtonTextures[self.__skin].up
	self.Left:SetTexture(up)
	self.Right:SetTexture(up)
	self.Middle:SetTexture(up)
end

local function NewButton(parent, skin, name)
	local button = CreateFrame("Button", name, parent or UIParent)
	button:SetSize(40, 22)

	local left = button:CreateTexture(nil, "BACKGROUND")
	left:SetPoint("TOPLEFT")
	left:SetPoint("BOTTOMLEFT")
	left:SetWidth(12)
	left:SetTexCoord(0, 12/128, 0, 22/32)
	button.Left = left

	local right = button:CreateTexture(nil, "BACKGROUND")
	right:SetPoint("TOPRIGHT")
	right:SetPoint("BOTTOMRIGHT")
	right:SetWidth(12)
	right:SetTexCoord(68/128, 80/128, 0, 22/32)
	button.Right = right

	local middle = button:CreateTexture(nil, "BACKGROUND")
	middle:SetPoint("TOPLEFT", left, "TOPRIGHT")
	middle:SetPoint("BOTTOMRIGHT", right, "BOTTOMLEFT")
	middle:SetTexCoord(12/128, 68/128, 0, 22/32)
	button.Middle = middle

	local text = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	text:SetAllPoints(true)
	text:SetJustifyH("CENTER")
	text:SetJustifyV("CENTER")
	button:SetFontString(text)
	button:SetNormalFontObject("GameFontNormal")
	button:SetHighlightFontObject("GameFontHighlight")	

	button:SetScript("OnMouseDown", OnMouseDown)
	button:SetScript("OnMouseUp", OnMouseUp)
	button:SetScript("OnShow", OnMouseUp)

	button.GetSkin = GetSkin
	button.SetSkin = SetSkin

	SetSkin(button, skin or "RED")

	return button
end

-- ****************************************************************************
-- * Misc Functions and Settings                                              *
-- ****************************************************************************
local CmdListButtons = {}
local DisplayedButtons = 0
local Selected = false
local OldSelected = false
local BottomButtons = {}
local NumberBottomButtons = 10
local oldPos = -1
local oldLinePixels= -1
local pos
local me

local screenPct = .9 -- percent of screen the window takes up

local Edit, Help, Transfer, AutoRuns, Settings, Mode = 1, 2, 3, 4, 5, 1
local prevMode = 0

local SMS = SlashMagic_Settings

local IndentationLib = AddonTable.IndentationLib
local ColorTable = AddonTable.defaultColorTable
-- Assign basic colors
do
	local T = IndentationLib.Tokens
	--- Assigns a color to multiple tokens at once.
	local function Color ( Code, ... )
		for Index = 1, select( "#", ... ) do
			ColorTable[ select( Index, ... ) ] = Code;
		end
	end
	Color( "|cff88bbdd", T.KEYWORD ); -- Reserved words
	Color( "|cffff6666", T.UNKNOWN );
	Color( "|cffcc7777", T.CONCAT, T.VARARG, T.ASSIGNMENT, T.PERIOD, T.COMMA, T.SEMICOLON, T.COLON, T.SIZE );
	Color( "|cffffaa00", T.NUMBER );
	Color( "|cff888888", T.STRING, T.STRING_LONG );
	Color( "|cff55cc55", T.COMMENT_SHORT, T.COMMENT_LONG );
	Color( "|cffccaa88", T.LEFTCURLY, T.RIGHTCURLY, T.LEFTBRACKET, T.RIGHTBRACKET, T.LEFTPAREN, T.RIGHTPAREN, T.ADD, T.SUBTRACT, T.MULTIPLY, T.DIVIDE, T.POWER, T.MODULUS );
	Color( "|cffccddee", T.EQUALITY, T.NOTEQUAL, T.LT, T.LTE, T.GT, T.GTE );
end

local ichars = function( value, block ) -- replace invisible chars in string
	value = value:gsub("|c%x%x%x%x%x%x%x%x", (block and "XCOLORCODE" or ""))
	value = value:gsub("|r", (block and "RR" or ""))
	return value
end

tempCommandStorage = {}
local tempCommandGrab = function()
	tempCommandStorage = {}
	local TCS = tempCommandStorage
	TCS.Selected = Selected
	TCS.desc = me.DescFrameEBox:GetText()
	TCS.vers = me.VersionText:GetText()
	TCS.code = me.CodeFrameEBox:GetText()
	TCS.aliases = me.AliasEBox:GetText()
	TCS.BFslider = me.BFslider:GetValue()
	TCS.cursorPos = me.CodeFrameEBox:GetCursorPosition()
	TCS.EBoxScroll = me.ScrollFrame:GetVerticalScroll()
	TCS.Full = true
end

local EDGE = 48 -- lines to keep around edge of screen in editor * 12 pixel font size

local HintSet = function(self) --sets the "OnEnter" hint 
	me.HintFrameText:SetText( self.HintText )
end
local BottomButtonHintSet = function(self)
	me.HintFrameText:SetText( SM.BottomButtonHints[self.Name] )
end
local HintClear = function() --clears hint on "OnLeave"
	me.HintFrameText:SetText( " " )
end

local Backdrop={ --Main Frames
	bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
	edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border",
	tile = true,
	tileSize = 16,
	edgeSize = 16,
	insets = {
		left = 0,
		right = 0,
		top = 0,
		bottom = 0
	}
}
local Backdrop2={ --Sub Frames
	bgFile = "",
	edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border",
	tile = true,
	tileSize = 16,
	edgeSize = 16,
	insets = {
		left = 0,
		right = 0,
		top = 0,
		bottom = 0
	}
}
local Backdrop3 = { --Edit Boxes
	bgFile="",
	edgeFile="Interface\\Tooltips\\UI-Tooltip-Border",
	tile="true",
	tileSize= 32,
	edgeSize=10,
	insets = {
		left=5,
		right=5,
		top=5,
		bottom=5
	}
}

local CmdListDisplayUpdate = {}

-- ****************************************************************************
-- * Edit Mode Setup

CmdListDisplayUpdate[Edit] = function()
	if Mode ~= Edit then return end
	if prevMode ~= Mode then
		
		me.CodeFrame.HintText = "Edit the code for your slash command here. Your function will be called as: function(msg)     For help with text colors and addon provided functions click the help tab above. When switching between tabs your code changes may be lost if you do not save them."

		me.EditMode:SetSkin("BLUE")
		me.HelpMode:SetSkin("RED")
		me.TransferMode:SetSkin("RED")
		me.AutoRunsMode:SetSkin("RED")
		me.SettingsMode:SetSkin("RED")
		
		local ButtonText = { "Active", 		"", 	"New", 		"Delete", 		"Del Saved Vars", 		"Default", 		"Restore", 		"Archive", 		"Restore from Archive", "Save"}
		local ButtonSkin = { "RED",			"GREY",	"RED",		"RED",			"RED",					"RED",			"RED",			"RED",			"RED",					"RED"}
		local ButtonKey  = { "EDIT_Active",	"",		"EDIT_New",	"EDIT_Delete",	"EDIT_DelSavedVars",	"EDIT_Default",	"EDIT_Restore",	"EDIT_Archive",	"EDIT_RestoreArchive",	"EDIT_Save"}
		local button
		for x = 1, NumberBottomButtons do
			button = me["BottomButton"..x]
			button:SetSkin(ButtonSkin[x])
			button:SetText(ButtonText[x])
			button.Name = ButtonKey[x]
			BottomButtons[ButtonKey[x]] = button
		end
		
		me.BFslider:SetValue(0)
		me.BFslider:Enable()
		me.ButtonFrame:EnableMouseWheel(true)
		
		--if tempCommandStorage.Full then
		--	tempCommandRestore()
		--end
		
		me.CodeFrame:Show()
		me.SettingsFrame:Hide()
		
		IndentationLib.Disable(me.CodeFrameEBox)
		IndentationLib.Enable(me.CodeFrameEBox, 5, ColorTable)
		me.CodeFrameEBox:SetText("")
		IndentationLib.Update(me.CodeFrameEBox)
		
		Selected = tempCommandStorage.Full and tempCommandStorage.Selected or false
		OldSelected = false
				
		prevMode = Mode
	end
	
	local list = {}
	local offset
	local size = 0
	
	for index in pairs( SlashMagic_Commands ) do
		if (SMS.ShowDead and SlashMagic_Commands[index].isDead) or not(SlashMagic_Commands[index].isDead) then
			list[#list + 1] = index
		end
	end
	table.sort( list )
	size = #list
	if not Selected then Selected = list[1] end
		
	if size <= DisplayedButtons then
		me.BFslider:SetValue(0)
		me.BFslider:Disable()
	else
		me.BFslider:Enable()
		me.BFslider:SetMinMaxValues( 0, size - DisplayedButtons)
	end
	
	if tempCommandStorage.Full then
		offset = tempCommandStorage.BFslider
	else
		offset = me.BFslider:GetValue()
	end
	
	for x = 1, DisplayedButtons do
		local which = x + offset
		local button = CmdListButtons[ x ]
		if which <= size then
			button:SetText(list[which])
			if SMS.ShowGems and SM.DefaultCmdsLoaded and SM.Commands_Defaults[list[which]] then
				if SM.Commands_Defaults[list[which]].isDead then
					button.Gem:SetTexture("Interface\\AddOns\\SlashMagicConfig\\DeadGem.tga")
				else
					button.Gem:SetTexture("Interface\\AddOns\\SlashMagicConfig\\DefaultGem.tga")
				end
			else
				button.Gem:SetTexture( nil )
			end
	
			if Selected == list[which] then -- selected command in editor
				button:SetSkin("BLUE")
				if SlashMagic_Commands[list[which]].isDead then
					BottomButtons["EDIT_Active"]:SetText("Dead")
					BottomButtons["EDIT_Active"]:SetSkin("BLACK")
				elseif SlashMagic_Commands[list[which]].state then
					BottomButtons["EDIT_Active"]:SetText("Active")
					BottomButtons["EDIT_Active"]:SetSkin("GREEN")
				else
					BottomButtons["EDIT_Active"]:SetText("Not Active")
					BottomButtons["EDIT_Active"]:SetSkin("RED")
				end
			elseif SlashMagic_Commands[ list[which] ].isDead then -- dead command
				button:SetSkin("BLACK")
			elseif SlashMagic_Commands[ list[which] ].state then -- enabled command
				button:SetSkin("GREEN")
			else -- disabled command
				button:SetSkin("RED")
			end
		else
			button:SetText("")
			button:SetSkin("GREY")
		end
	end
	
	if SM.DefaultCmdsLoaded and SM.Commands_Defaults[Selected] and SlashMagic_Commands[Selected].vers == SM.Commands_Defaults[Selected].vers then
		BottomButtons["EDIT_Default"]:SetSkin("RED")
	elseif SM.DefaultCmdsLoaded and SM.Commands_Defaults[Selected] then
		BottomButtons["EDIT_Default"]:SetSkin("GREEN")
	else
		BottomButtons["EDIT_Default"]:SetSkin("GREY")
	end
	
	if OldSelected ~= Selected then
		--if tempCommandStorage.Full then
		--	tempCommandRestore()
		--end
		OldSelected = Selected
		if tempCommandStorage.Full then
			local TCS = tempCommandStorage
			me.NameFrameText:SetText( ("Name: %s"):format(TCS.Selected) )
			me.VersionText:SetText(TCS.vers)
			me.AliasEBox:SetText( TCS.aliases)
			me.DescFrameEBox:SetText( TCS.desc )
			me.CodeFrameEBox:SetText( TCS.code )
			me.BFslider:SetValue(TCS.BFslider)
			tempCommandStorage.Full = false
		else
			local command = SlashMagic_Commands[Selected]
			me.NameFrameText:SetText( ("Name: %s"):format(Selected) )
			local whatisit = type( command.vers )
			if whatisit == "number" then
				me.VersionText:SetText(("Version: %.2f"):format(command.vers))
			elseif whatisit == "string" then
				me.VersionText:SetText(("Version: %s"):format(command.vers))
			else
				me.VersionText:SetText(" ")
			end
			me.AliasEBox:SetText( command.alias and command.alias or "")
			me.DescFrameEBox:SetText( command.desc )
			local tempCode = string.join("\r",unpack(command.code))
			tempCode = tempCode:gsub("|", "||") -- escape pipes so pipe codes in code will display in edit box
			me.CodeFrameEBox:SetText( tempCode )
		end
	end
end

-- ****************************************************************************
-- * Help screen Setup

CmdListDisplayUpdate[Help]= function()	
	if prevMode ~= Mode then
		if prevMode == Edit and not(tempCommandStorage.Full) then
			tempCommandGrab()
		end
		
		me.CodeFrame.HintText = "These functions and tables will help you with your data management, event handling, and more. When switching between tabs your code changes may be lost if not saved"
	
		me.EditMode:SetSkin("RED")
		me.HelpMode:SetSkin("BLUE")
		me.TransferMode:SetSkin("RED")
		me.AutoRunsMode:SetSkin("RED")
		me.SettingsMode:SetSkin("RED")
	
--		Selected = false
--		OldSelected = false

		local ButtonText = { "", 		"", 	"", 	"", 	"", 	"", 	"", 	"", 	"", 	""}
		local ButtonSkin = { "GREY",	"GREY",	"GREY",	"GREY",	"GREY",	"GREY",	"GREY",	"GREY",	"GREY",	"GREY"}
		local ButtonKey =  { "",		"",		"",		"",		"",		"",		"",		"",		"",		""}
		local button
		for x = 1, NumberBottomButtons do
			button = me["BottomButton"..x]
			button:SetSkin(ButtonSkin[x])
			button:SetText(ButtonText[x])
			button.Name = ButtonKey[x]
			BottomButtons[ButtonKey[x]] = button
		end
		me.BFslider:SetValue(0)
		me.BFslider:Disable()
		me.ButtonFrame:EnableMouseWheel(false)
		
		IndentationLib.Disable(me.CodeFrameEBox)
		
		me.NameFrameText:SetText("Helpful Addon assistance")
		me.VersionText:SetText("")
		me.AliasEBox:SetText("")
		me.DescFrameEBox:SetText( "provided addon help located here" )
		
		me.CodeFrame:Show()
		me.SettingsFrame:Hide()
		
		for x = 1, DisplayedButtons do
			CmdListButtons[ x ]:SetText("")
			CmdListButtons[ x ]:SetSkin("GREY")
			CmdListButtons[ x ].Gem:SetTexture()
		end
		
		prevMode = Mode
	end
	

-- ****************************************************************************
-- * Command Help Text

	local header = {
		"These Variable tables are provided to assist with persistence and scope",
		"    "..color.Blue.."static "..color.Reset.." = per session variables that remain even when your command ends",
		"",
		"    "..color.Blue.."saved  "..color.Reset.." = saved variables that will be there through sessions and reloads",
		"",
		"    "..color.Blue.."savedPC"..color.Reset.." = Per Character saved variables that will be there through sessions and reloads",
		"",
		"",
		"The color table is available to make your text output look nicer",
		"    color:dRGBA(color name from below) returns a table with decimal rgba values ( 0 - 255)",
		"    color:fRGBA(color name from below) returns a table with fractional rgba values (0.0 - 1.0)",
		"    color.ColorName is the text pipe code for the specified color (includes pipes and all)",
		"    The standard HTML names are supported/used",
		"",
	}
	
	local colorcodes = {}
	local names = {}
	local maxNameWidth = 0
	for index, value in pairs(color) do
		if type(value) == "string" and value ~= "Reset" then
			names[#names + 1] = index
			maxNameWidth = math.max(maxNameWidth, index:len())
		end
	end
	
	table.sort(names)
	
	local c1, c2, c3, c4, c5
	local cc1, cc2, cc3, cc4, cc5
	local s1, s2, s3, s4, s5
	
	maxNameWidth = maxNameWidth + 2
	
	for x = 1, #names, 5 do
		c1 = names[x] and names[x] or ""
		cc1 = names[x] and color[c1] or ""
		s1 = string.rep(" ", maxNameWidth - c1:len())
		
		c2 = names[x + 1] and names[x + 1] or ""
		cc2 = names[x + 1] and color[c2] or ""
		s2 = string.rep(" ", maxNameWidth - c2:len())
		
		c3 = names[x + 2] and names[x + 2] or ""
		cc3 = names[x + 2] and color[c3] or ""
		s3 = string.rep(" ", maxNameWidth - c3:len())
		
		c4 = names[x + 3] and names[x + 3] or ""
		cc4 = names[x + 3] and color[c4] or ""
		s4 = string.rep(" ", maxNameWidth - c4:len())
		
		c5 = names[x + 4] and names[x + 4] or ""
		cc5 = names[x + 4] and color[c5] or ""
		s5 = string.rep(" ", maxNameWidth - c5:len())
		
		colorcodes[#colorcodes + 1] = 
		("    %s%s|r%s %s%s|r%s %s%s|r%s %s%s|r%s %s%s|r%s"):format(cc1, c1, s1, cc2, c2, s2, cc3, c3, s3, cc4, c4, s4, cc5, c5, s5)
	end
	
	local footer = {
		"    Reset (returns color to normal)",
		"",
		"",
		"These Functions are here to help reduce your coding needs",
		"    "..color.Blue.."OnEvent"..color.Reset.." = "..color.Green.."OnEvent(\"EventName\", function)"..color.Reset, --SlashMagic_OnEvent = function(command, event, doWhat)
		"              "..color.Green.."OnEvent(\"EventName\", nil)"..color.Reset,
		"              When called it will register an event to a frame to do your code when the event triggers",
		"              To unregister the event call again with nil instead of a function.",
		"              You can have as many events as you wish but are all defined seperatly and only 1 per event",
		"",
		"    "..color.Blue.."InTime "..color.Reset.. " = "..color.Green.."InTime(firstRunTime, repeatTime, function)"..color.Reset, --function(command, seconds, again, doWhat)
		"              "..color.Green.."InTime(firstRunTime, boolean, function)"..color.Reset,
		"              Will run the function after firstRunTime delay in seconds",
		"              If repeat time is passed it will repeat the function in intervals of repeatTime",
		"              If boolean is passed as true it will repeat the function every firstRunTime",
		"              If boolean is passed as false it will not repeat the function",
		"              To stop a repeating function you need to return true and it will stop and self delete itself",
		"",
		"    "..color.Blue.."DoIt   "..color.Reset.." = "..color.Green.."DoIt(\"/SlashCommand and arguments\")"..color.Reset,--function("/slashstring")
		"              Will attempt to run the passed slash command with arguments as you can no longer use the Macro Edit Box to run them",
	}
	
	
	local tempCode = string.join("\r",unpack(header))
	tempCode = tempCode.."\r"..string.join("\r", unpack(colorcodes))
	tempCode = tempCode.."\r"..string.join("\r", unpack(footer))
	--tempCode = tempCode:gsub("|", "||") -- escape pipes so pipe codes in code will display in edit box
	me.CodeFrameEBox:SetText( tempCode )
end


-- ****************************************************************************
-- * Auto Run Setup

CmdListDisplayUpdate[AutoRuns] = function()	
	if prevMode ~= Mode then
		if prevMode == Edit and not(tempCommandStorage.Full) then
			tempCommandGrab()
		end
		me.EditMode:SetSkin("RED")
		me.HelpMode:SetSkin("RED")
		me.TransferMode:SetSkin("RED")
		me.AutoRunsMode:SetSkin("BLUE")
		me.SettingsMode:SetSkin("RED")
	
--		Selected = false
--		OldSelected = false
		local ButtonText = { "Active", 			"", 	"", 	"", 	"", 	"", 	"Run Now", 			"", 	"Restore", 			"Save"}
		local ButtonSkin = { "RED",				"GREY",	"GREY",	"GREY",	"GREY",	"GREY",	"RED",				"GREY",	"RED",				"RED"}
		local ButtonKey =  { "AUTORUN_Active",	"",		"",		"",		"",		"",		"AUTORUN_RunNow",	"",		"AUTORUN_Restore",	"AUTORUN_Save"}
		local button
		for x = 1, NumberBottomButtons do
			button = me["BottomButton"..x]
			button:SetSkin(ButtonSkin[x])
			button:SetText(ButtonText[x])
			button.Name = ButtonKey[x]
			BottomButtons[ButtonKey[x]] = button
		end
		me.BFslider:SetValue(0)
		me.BFslider:Disable()
		me.ButtonFrame:EnableMouseWheel(false)
		
		me.CodeFrame:Show()
		me.SettingsFrame:Hide()
		
		IndentationLib.Disable(me.CodeFrameEBox)
		
		me.NameFrameText:SetText("Configure Auto-Run Commands")
		me.VersionText:SetText("")
		me.AliasEBox:SetText("")
		me.DescFrameEBox:SetText( "Setup slash commands to run after you login" )
		
		me.CodeFrame.HintText = "Put slash commands to be auto ran in the list above. Commands created by this addon must be referred to by created command name NOT by an alias to ensure they can be ran. Not all commands can be ran automatically at login and they may throw an error"
		if not(SMS.AutoRuns) then
			SMS.AutoRuns = {}
		end
		local tempCode = string.join("\r\n",unpack(SMS.AutoRuns))
		me.CodeFrameEBox:SetText( tempCode and tempCode or "" )
		
		for x = 1, DisplayedButtons do
			CmdListButtons[ x ]:SetText("")
			CmdListButtons[ x ]:SetSkin("GREY")
			CmdListButtons[ x ].Gem:SetTexture( nil )
		end
		
		
		
		prevMode = Mode
	end
	
	if SMS.EnableAutoRuns then
		BottomButtons["AUTORUN_Active"]:SetText("Active")
		BottomButtons["AUTORUN_Active"]:SetSkin("GREEN")
	else
		BottomButtons["AUTORUN_Active"]:SetText("Not Active")
		BottomButtons["AUTORUN_Active"]:SetSkin("RED")
	end
end

-- ****************************************************************************
-- * Settings Setup

CmdListDisplayUpdate[Settings]= function()	
	if prevMode ~= Mode then
		if prevMode == Edit and not(tempCommandStorage.Full) then
			tempCommandGrab()
		end
		me.EditMode:SetSkin("RED")
		me.HelpMode:SetSkin("RED")
		me.TransferMode:SetSkin("RED")
		me.AutoRunsMode:SetSkin("RED")
		me.SettingsMode:SetSkin("BLUE")
	
--		Selected = false
--		OldSelected = false
		local ButtonText = { "", 		"", 	"", 	"", 	"", 	"", 	"", 	"", 	"", 	""}
		local ButtonSkin = { "GREY",	"GREY",	"GREY",	"GREY",	"GREY",	"GREY",	"GREY",	"GREY",	"GREY",	"GREY"}
		local ButtonKey =  { "",		"",		"",		"",		"",		"",		"",		"",		"",		""}
		local button
		for x = 1, NumberBottomButtons do
			button = me["BottomButton"..x]
			button:SetSkin(ButtonSkin[x])
			button:SetText(ButtonText[x])
			button.Name = ButtonKey[x]
			BottomButtons[ButtonKey[x]] = button
		end
		me.BFslider:SetValue(0)
		me.BFslider:Disable()
		me.ButtonFrame:EnableMouseWheel(false)
		
		IndentationLib.Disable(me.CodeFrameEBox)
		
		me.NameFrameText:SetText("Configure Addon Settings")
		me.VersionText:SetText("")
		me.AliasEBox:SetText("")
		me.DescFrameEBox:SetText( "Change Addon Settings Here" )
		
		me.CodeFrame:Hide()
		me.SettingsFrame:Show()
		
		for x = 1, DisplayedButtons do
			CmdListButtons[ x ]:SetText("")
			CmdListButtons[ x ]:SetSkin("GREY")
			CmdListButtons[ x ].Gem:SetTexture()
		end
		
		prevMode = Mode
	end
	me.EnableAddon:SetChecked(SMS.Enable)
	me.LoadDefaultCommands:SetChecked(SMS.LoadDefaultCommands)
	me.ShowGems:SetChecked(SMS.ShowGems)
	me.ShowDead:SetChecked(SMS.ShowDead)
	me.EnableAutoRuns:SetChecked(SMS.EnableAutoRuns)
	me.EnableArchives:SetChecked(SMS.EnableArchives)
end

-- ****************************************************************************
-- * Transfer Setup

CmdListDisplayUpdate[Transfer] = function()	
	if prevMode ~= Mode then
		if prevMode == Edit and not(tempCommandStorage.Full) then
			tempCommandGrab()
		end
		me.EditMode:SetSkin("RED")
		me.HelpMode:SetSkin("RED")
		me.TransferMode:SetSkin("BLUE")
		me.AutoRunsMode:SetSkin("RED")
		me.SettingsMode:SetSkin("RED")
		
		local ButtonText = { "", 	 "", 	 "", 	 "", 	 "", 	 "", 	 "", 	 "", 	 "", 	 ""}
		local ButtonSkin = { "GREY", "GREY", "GREY", "GREY", "GREY", "GREY", "GREY", "GREY", "GREY", "GREY"}
		local ButtonKey =  { "",	 "",	 "",	 "",	 "",	 "",	 "",	 "",	 "",	 ""}
		local button
		for x = 1, NumberBottomButtons do
			button = me["BottomButton"..x]
			button:SetSkin(ButtonSkin[x])
			button:SetText(ButtonText[x])
			button.Name = ButtonKey[x]
			BottomButtons[ButtonKey[x]] = button
		end
		
		me.CodeFrame:Show()
		me.SettingsFrame:Hide()
		
		prevMode = Mode
	end
end

	
local acceptDialogHandler = {
	["DELETE"] = function() -- Deletes Selected command
		SM.Disable(Selected)
		SlashMagic_Commands[Selected] = nil
		SlashMagic_CmdVars[Selected] = nil
		SlashMagic_CmdVarsPC[Selected] = nil
		Selected = false
	end,
	
	["DELETE DEFAULT ENABLED"] = function()
		SlashMagic_Commands[Selected].state = false
		SM.Disable(Selected)
	end,
	
	["DELETE SAVED VARS"] = function()
		if SlashMagic_CmdVars[Selected] then
			table.wipe(SlashMagic_CmdVars[Selected])
		end
		if SlashMagic_CmdVarsPC[Selected] then
			table.wipe(SlashMagic_CmdVarsPC[Selected])
		end
		if SlashMagicStaticValues[Selected] then
			table.wipe(SlashMagicStaticValues[Selected])
		end
	end,
	
	["RESET ADDON"] = function() -- Resets Addon Back To "Factory"
		SM.Reset()
		Selected = false
	end,
	
	["DEFAULT COMMAND"] = function() -- Restores selected command to "factory default"
		SM.Reset1( Selected )
		OldSelected = false
	end,
		
	["RESTORE"] = function() -- Restore to state before last save
		OldSelected = false
	end,
		
	["RESTORE AUTORUNS"] = function() -- Restore Auto Run data in editbox
		local tempCode = string.join("\r",unpack(SMS.AutoRuns))
		me.CodeFrameEBox:SetText( tempCode and tempCode or "" )
	end,
	
	["LOAD DEFAULT COMMANDS"] = function(self) -- toggle loading of default commands (button on settings page)
		SMS.LoadDefaultCommands = not(SMS.LoadDefaultCommands)
		CmdListDisplayUpdate[Mode]()
		self.text:SetText("Reload the UI now?")
		self.button1:SetText("Reload")
		self.data = "RELOAD"
		return true
	end,
	
	["RELOAD"] = ReloadUI,
}

StaticPopupDialogs["SLASHMAGIC"] = {
	text = "%s",
	button1 = "Yes",
	button2 = "Cancel",
	OnAccept = function( self, data, data2)
		local keep = acceptDialogHandler[data](self)
		CmdListDisplayUpdate[Mode]()
		return keep
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	enterClicksFirstButton = true,
	preferredIndex = 3,  -- avoid some UI taint, see http://www.wowace.com/announcements/how-to-avoid-some-ui-taint/ 
}

SM.BottomButtonHints = {
	[""] = "",
	-- Auto Run Bottom Button Hints
	["AUTORUN_Active"] 		= "This button enables/disables autorun functionality",
	["AUTORUN_RunNow"]		= "This button will save the autorun commands and then run them. (Even if not set to active.)",
	["AUTORUN_Restore"] 	= "This button will restore Autoruns to last save state",
	["AUTORUN_Save"] 		= "This button will save the current AutoRun Script",
	-- Edit Mode Bottom Button HintSet
	["EDIT_Active"] 		= "This button enables / disables the slash command. Green is enabled, Red is disabled",
	["EDIT_New"]			= "This button allows you to make a new slash command",
	["EDIT_Delete"]			= "This button deletes the selected (non default) slash command",
	["EDIT_DelSavedVars"]	= "This Button will delete any saved variables for the selected command",
	["EDIT_Default"]		= "This button will restore an edited default command, It will turn green when a default command is selected.",
	["EDIT_Restore"]		= "This button will restore an edited command back to the state of its last save",
	["EDIT_Archive"]		= "Archive a copy of the command incase it no longer comes as a default command (Disableing will delete any archive already made)",
	["EDIT_RestoreArchive"] = "Restore the saved archive of this command (The command will auto-restore if removed from command defaults)",
	["EDIT_Save"]			= "This button will save the above description/code for the selected command",
}

local BottomButtonHandler = {}
BottomButtonHandler[""] = function() end
	
-- ************************************************************************
-- Auto Run Bottom Buttons
BottomButtonHandler["AUTORUN_Active"] = function()
	SMS.EnableAutoRuns = not(SMS.EnableAutoRuns)
end
	
BottomButtonHandler["AUTORUN_RunNow"] = function()
	BottomButtonHandler["AUTORUN_Save"]()
	print( "|cFF00FF00Slash Magic|r auto-run commands are now being run. (Manually Triggered)")
	for x = 1, #SMS.AutoRuns do
		if SMS.AutoRuns[x] ~= "" then
			Wait( (x - 1) * 5, DoIt, SMS.AutoRuns[x] )
		end
	end
end
	
BottomButtonHandler["AUTORUN_Restore"] = function()
	local dialog = StaticPopup_Show("SLASHMAGIC", "Are you sure you want to discard all of your changes to the\"|cFF00FFFFAuto-Runs|r\" section since the last save?" )
	dialog.data = "RESTORE AUTORUNS"
	dialog.button1:SetText("Discard")
	dialog.button1:Enable()
end
	
BottomButtonHandler["AUTORUN_Save"] = function()
	local tempCode = {string.split("\n", me.CodeFrameEBox:GetText()) }
	for x = 1, #tempCode do
		tempCode[x] = tempCode[x]:gsub("\n", "")
		tempCode[x] = tempCode[x]:gsub("\r", "")
		tempCode[x] = string.trim(tempCode[x])
		if (tempCode[x]:len() == 0) then
			table.remove(tempCode, x)
		end
	end
	
	SMS.AutoRuns = {unpack(tempCode)}
end


-- ************************************************************************
-- Edit Mode Bottom Buttons
BottomButtonHandler["EDIT_Active"] = function()
	SlashMagic_Commands[Selected].state = SlashMagic_Commands[Selected].state == false
	if SlashMagic_Commands[Selected].state then
		SlashMagic_Commands[Selected].state = SM.Enable( Selected )
	else
		SM.Disable( Selected )
	end
end

BottomButtonHandler["EDIT_New"]= function()
	local dialog = StaticPopup_Show("SLASHMAGIC_NEWCOMMAND")
	dialog.editBox:SetText("")
	dialog.editBox:SetFocus()
end
	
BottomButtonHandler["EDIT_Delete"] = function()
	local dialog
	if SM.DefaultCmdsLoaded and SM.Commands_Defaults[Selected] and SlashMagic_Commands[Selected].state then
		dialog = StaticPopup_Show("SLASHMAGIC", ("You cannot delete the command: \"|cFF00FFFF%s|r\" as it is a default command. Would you like to disable it"):format(Selected))
		dialog.button1:SetText("Disable")
		dialog.button1:Enable()
		dialog.data = "DELETE DEFAULT ENABLED"
	elseif SM.DefaultCmdsLoaded and SM.Commands_Defaults[Selected] and not(SlashMagic_Commands[Selected].state) then
		dialog = StaticPopup_Show("SLASHMAGIC", ("You cannot delete the command: \"|cFF00FFFF%s|r\" as it is a default command. The command is already disabled as well."):format(Selected))
		dialog.button1:SetText("Disable")
		dialog.button1:Disable()
		dialog.data = "DELETE DEFAULT DISABLED"
	else
		dialog = StaticPopup_Show("SLASHMAGIC", ("Are you sure you want to delete the selected command: \"|cFF00FFFF%s|r\" ?"):format(Selected))
		dialog.data = "DELETE"
		dialog.button1:SetText("Delete")
		dialog.button1:Enable()
	end
end

BottomButtonHandler["EDIT_DelSavedVars"] = function()
	local dialog = StaticPopup_Show("SLASHMAGIC", ("Do you wish to delete the saved and static variables for command: \"|cFF00FFFF%s|r\""):format(Selected))
	dialog.button1:SetText("Delete")
	dialog.button1:Enable()
	dialog.data = "DELETE SAVED VARS"
end

BottomButtonHandler["EDIT_Default"] = function()
	local dialog = StaticPopup_Show("SLASHMAGIC", ("Do you want to restore \"|cFF00FFFF%s|r\" back to \"Factory\" default?"):format( Selected ))
	dialog.data = "DEFAULT COMMAND"
	dialog.button1:SetText("Restore")
	dialog.button1:Enable()
end

BottomButtonHandler["EDIT_Restore"] = function()
	local dialog = StaticPopup_Show("SLASHMAGIC", ("Are you sure you want to discard all of your changes to \"|cFF00FFFF%s|r\" since the last save?" ):format(Selected))
	dialog.data = "RESTORE"
	dialog.button1:SetText("Discard")
	dialog.button1:Enable()
end
	
BottomButtonHandler["EDIT_Archive"] = function()
end
	
BottomButtonHandler["EDIT_RestoreArchive"] = function()
end

BottomButtonHandler["EDIT_Save"] = function()
	local command = SlashMagic_Commands[Selected]
	command.desc = me.DescFrameEBox:GetText()
	command.vers = "Custom"
	local tempCode = me.CodeFrameEBox:GetText()
	tempCode = tempCode:gsub("||", "|") -- remove escape of the pipe char so codes will work
	command.code = {string.split("\n\r", tempCode ) }
	local aliases = me.AliasEBox:GetText()
	aliases = string.trim(aliases)
	aliases = string.gsub( aliases, "%s%s+", " ")
	if aliases == " " or aliases == "" then
		command.alias = nil
	else
		command.alias = aliases
	end
	SM.Disable(Selected)
	SM.Enable(Selected)
	OldSelected = false
	CmdListDisplayUpdate[Mode]()
end


-- ****************************************************************************
-- * Frame begins here                                                        *
-- ****************************************************************************

SM.ConfigFrame = CreateFrame("Frame" , "SMConfigFrame", UIParent, "BackdropTemplate" )
me = SM.ConfigFrame
me:SetFrameStrata("DIALOG")
me:Hide()
me:EnableMouse(true)
me:SetPoint("CENTER", UIParent, "CENTER")
me:SetHeight( screenPct * GetScreenHeight() )
me:SetWidth( screenPct * GetScreenWidth() )
me:SetBackdrop( Backdrop )
me:SetBackdropColor( 128, 128,128, 1)
me:SetBackdropBorderColor( 255, 255, 255, 1)
me:SetMovable(true)
me:SetUserPlaced(false)
me:SetClampedToScreen(true)

me:SetScript("OnShow", function(self)
	if DisplayedButtons == 0 then
		local MaxButtons = 0
		local BHeight = 0
		local BFHeight = me.ButtonFrame:GetHeight()
		local whatsleft
		local PrevButton
		
		repeat
			DisplayedButtons = DisplayedButtons + 1
			CmdListButtons[DisplayedButtons] = NewButton( me.ButtonFrame, "RED", "SMCmdListButton"..DisplayedButtons )
			local Button = CmdListButtons[DisplayedButtons]
			Button.Gem = Button:CreateTexture(nil, "OVERLAY")
			Button.Gem:SetPoint("RIGHT", Button, "LEFT", 4, 0)
			Button.Gem:SetHeight(BHeight*.7)
			local fontString = Button:GetFontString()
			fontString:SetJustifyH("LEFT")
			fontString:ClearAllPoints()
			fontString:SetPoint("TOPRIGHT", Button, "TOPRIGHT")
			fontString:SetPoint("BOTTOMLEFT", Button, "BOTTOMLEFT", 7, 0)
			if DisplayedButtons == 1 then
				Button:SetPoint("TOPLEFT", me.ButtonFrame,  "TOPLEFT", 5, -5)
				BHeight = Button:GetHeight() - 1
				MaxButtons = math.floor(BFHeight / BHeight)
				whatsleft = BFHeight % BHeight
			else
				Button:SetPoint("TOP", PrevButton,  "BOTTOM", 0, 1 )
			end
			Button.Gem:SetHeight(BHeight - 2)
			Button.Gem:SetWidth(BHeight - 2)
			Button:SetWidth( 140 )
			Button:SetScript("OnClick", function(self, button, down)
				if button ~= "LeftButton" or self:GetSkin() == "GREY" then
					return
				end
				PlaySound( 624 ) -- "GAMEGENERICBUTTONPRESS"
				local text = self:GetText()
				if text ~= "" and text ~= nil then
					Selected = text
					CmdListDisplayUpdate[Mode]()
				end
				return
			end)
			Button.HintText = "Select the command you would like to edit, or scroll the list with your mousewheel. (Holding shift will scroll 5x faster)"
			Button:SetScript("OnEnter", HintSet )
			Button:SetScript("OnLeave", HintClear )
			PrevButton = Button
		until DisplayedButtons == MaxButtons
		me:SetHeight(( me:GetHeight() - whatsleft)  + ( BHeight / 2) + 1)
		CmdListDisplayUpdate[Mode]()
		me.BFslider:SetValue(0)
		
		local BottomButtonWidth = me.HintFrame:GetWidth() / NumberBottomButtons
		
		for x = 1, NumberBottomButtons do
			me["BottomButton"..x]:SetWidth(BottomButtonWidth)
		end
	end

	me.StrLen:SetWidth( me.EBoxOverlay:GetWidth() - 12)
	me.CodeFrameEBox:SetWidth( me.EBoxOverlay:GetWidth() - 3)
end)

me:RegisterEvent("PLAYER_REGEN_DISABLED")
me:SetScript("OnEvent", function(self, event, ...)
	if event == "PLAYER_REGEN_DISABLED" then
		self:Hide()
	end
end)

-- Title region
me.TitleRegion = CreateFrame("Frame", nil, me, "BackdropTemplate") -- ***
me.TitleRegion:SetPoint( "CENTER", me, "TOP", 0, -8)
me.TitleRegion:SetFrameStrata('DIALOG')
me.TitleRegion:SetToplevel(true)
me.TitleRegion:EnableMouse(true)
me.TitleRegion:SetMovable(true)
me.TitleRegion:RegisterForDrag("LeftButton" ) --, "RightButton") --Enable right button when resizeable
me.TitleRegion:SetClampedToScreen(true)		
me.TitleRegion:SetBackdrop(	{		-- ***
	bgFile='Interface\\FrameGeneral\\UI-Background-Marble' ,
	edgeFile='Interface\\DialogFrame\\UI-DialogBox-Border',
	tile = true,
	insets = {left = 0, right = 0, top = 0, bottom = 0},
	tileSize =  16,
	edgeSize =  16,
	}
)
me.TitleRegion:SetScript('OnDragStart', function(self, button)
	if not(self.parent) then
		self.parent = self:GetParent()
	end
	if button == "LeftButton" then
		self.parent:StartMoving()
		self.isMoving = true
		self.hasMoved = false
	elseif button == "RightButton" then
		self.parent:StartSizing()
		self.isMoving = true
		self.hasMoved = false
	end
end)

me.TitleRegion:SetScript('OnDragStop', function(self)
	if ( self.isMoving ) then
        self.parent:StopMovingOrSizing();
        self.isMoving = false;
        self.hasMoved = true;
    end
end)


-- Title frame text
me.Title = me.TitleRegion:CreateFontString( nil , "OVERLAY", "GameFontHighlight")
me.Title:SetText("Slash Magic Configuration")
me.Title:SetPoint( "CENTER", me.TitleRegion, "CENTER", 0, 0)
me.Title:SetJustifyH( "CENTER" )
me.Title:SetJustifyV( "MIDDLE" )

me.TitleRegion:SetWidth(me.Title:GetStringWidth() + 16)
me.TitleRegion:SetHeight(me.Title:GetStringHeight() + 16)

me.ModeFrame = CreateFrame("Frame" , "SMNameFrame", me, "BackdropTemplate" )
local ModeFrame = me.ModeFrame
ModeFrame:SetBackdrop( Backdrop2 )
ModeFrame:SetBackdropColor( 128, 128,128, 1)
ModeFrame:SetBackdropBorderColor( 255, 255, 255, 1)
ModeFrame:SetPoint( "TOPLEFT", me, "TOPLEFT", 10, -25 )
ModeFrame:SetHeight( 31 )
ModeFrame:SetPoint( "RIGHT", me, "RIGHT", -10, 0)

me.EditMode = NewButton( me, "GREEN", "SMEditModeButton" )
local EditMode = me.EditMode
EditMode:SetText("Edit")
EditMode.HintText = "This button switches to command edit mode"
EditMode:SetScript("OnEnter", HintSet )
EditMode:SetScript("OnLeave", HintClear )
EditMode:SetPoint("TOPLEFT", ModeFrame, "TOPLEFT", 2, -2)
EditMode:SetSize( 150, 25)
EditMode:SetScript("OnClick", function(self, button, down)
	if button ~= "LeftButton" or self:GetSkin() == "GREEN" then
		return
	end
	PlaySound( 624 ) -- "GAMEGENERICBUTTONPRESS"
	Mode = Edit
	CmdListDisplayUpdate[Mode]()
end)

me.HelpMode = NewButton( me, "GREEN", "SMHelpModeButton" )
local HelpMode = me.HelpMode
HelpMode:SetText("Code Help")
HelpMode.HintText = "This button switches to code help mode to show addon built in commands"
HelpMode:SetScript("OnEnter", HintSet )
HelpMode:SetScript("OnLeave", HintClear )
HelpMode:SetPoint("TOPLEFT", EditMode, "TOPRIGHT", 0, 0)
HelpMode:SetSize( 150, 25)
HelpMode:SetScript("OnClick", function(self, button, down)
	if button ~= "LeftButton" or self:GetSkin() == "GREEN" then
		return
	end
	PlaySound( 624 ) -- "GAMEGENERICBUTTONPRESS"
	Mode = Help
	CmdListDisplayUpdate[Mode]()
end)

me.TransferMode = NewButton( me, "RED", "SMTransferModeButton" )
local TransferMode = me.TransferMode
TransferMode:SetText("Transfer")
TransferMode.HintText = "This button switches to command transfer/share mode"
TransferMode:SetScript("OnEnter", HintSet )
TransferMode:SetScript("OnLeave", HintClear )
TransferMode:SetPoint("TOPLEFT", HelpMode, "TOPRIGHT", 0, 0)
TransferMode:SetSize( 150, 25)
TransferMode:SetScript("OnClick", function(self, button, down)
	if button ~= "LeftButton" or self:GetSkin() == "GREEN" then
		return
	end
	PlaySound( 624 ) -- "GAMEGENERICBUTTONPRESS"
	Mode = Transfer
	CmdListDisplayUpdate[Mode]()
end)

me.AutoRunsMode = NewButton( me, "RED", "SMAutoRunsModeButton" )
local AutoRunsMode = me.AutoRunsMode
AutoRunsMode:SetText("AutoRuns")
AutoRunsMode.HintText = "This button switches to AutoRuns edit mode"
AutoRunsMode:SetScript("OnEnter", HintSet )
AutoRunsMode:SetScript("OnLeave", HintClear )
AutoRunsMode:SetPoint("TOPLEFT", TransferMode, "TOPRIGHT", 0, 0)
AutoRunsMode:SetSize( 150, 25)
AutoRunsMode:SetScript("OnClick", function(self, button, down)
	if button ~= "LeftButton" or self:GetSkin() == "GREEN" then
		return
	end
	PlaySound( 624 ) -- "GAMEGENERICBUTTONPRESS"
	Mode = AutoRuns
	CmdListDisplayUpdate[Mode]()
end)

me.SettingsMode = NewButton( me, "RED", "SMSettingsModeButton" )
local SettingsMode = me.SettingsMode
SettingsMode:SetText("Settings")
SettingsMode.HintText = "This button switches to Settings mode"
SettingsMode:SetScript("OnEnter", HintSet )
SettingsMode:SetScript("OnLeave", HintClear )
SettingsMode:SetPoint("TOPLEFT", AutoRunsMode, "TOPRIGHT", 0, 0)
SettingsMode:SetSize( 150, 25)
SettingsMode:SetScript("OnClick", function(self, button, down)
	if button ~= "LeftButton" or self:GetSkin() == "GREEN" then
		return
	end
	PlaySound( 624 ) -- "GAMEGENERICBUTTONPRESS"
	Mode = Settings
	CmdListDisplayUpdate[Mode]()
end)

me.Close = NewButton( me, "RED", "SMCloseButton" )
local Close = me.Close
Close:SetText("Close")
Close.HintText = "This button closes the edit window"
Close:SetScript("OnEnter", HintSet )
Close:SetScript("OnLeave", HintClear )
Close:SetPoint("TOPRIGHT", ModeFrame, "TOPRIGHT", -2, -2 )
Close:SetSize( 150, 25)
Close:SetScript("OnClick", function(self, button, down)
	if button ~= "LeftButton" or self:GetSkin() == "GREY" then
		return
	end
	PlaySound( 624 ) -- "GAMEGENERICBUTTONPRESS"
	Mode = Edit
	me:Hide()
end)


-- Button Frame for command list buttons
me.ButtonFrame = CreateFrame("Frame" , "SMCommandButtonFrame", me, "BackdropTemplate" )
local ButtonFrame = me.ButtonFrame
ButtonFrame:SetBackdrop( Backdrop2 )
ButtonFrame:SetBackdropColor( 128, 128,128, 1)
ButtonFrame:SetBackdropBorderColor( 255, 255, 255, 1)
ButtonFrame:SetPoint( "TOPLEFT", ModeFrame, "BOTTOMLEFT", 0, -8 )
ButtonFrame:SetPoint( "BOTTOMLEFT", me, "BOTTOMLEFT", 10, 100 )
ButtonFrame:SetWidth(165)
ButtonFrame:EnableMouseWheel(1)

ButtonFrame:SetScript("OnMouseWheel", function(self, delta)
	SliderMW(me.BFslider, -delta)
end)

ButtonFrame.HintText = "Select the command you would like to edit, or scroll the list with your mousewheel. (Holding shift will scroll 5x faster)"
ButtonFrame:SetScript("OnEnter", HintSet )
ButtonFrame:SetScript("OnLeave", HintClear )

-- Slider for button list
me.BFslider = CreateFrame( "Slider", "SMBFSlider", me, "OptionsSliderTemplate" )
local BFslider = me.BFslider
BFslider:SetMinMaxValues( 0, 1 )
BFslider:SetOrientation("VERTICAL")
BFslider:SetPoint("TOPRIGHT", me.ButtonFrame ,"TOPRIGHT", -4, 0)
BFslider:SetPoint("BOTTOM", me.ButtonFrame ,"BOTTOM" )
BFslider:SetValueStep( 1 )
BFslider:SetWidth( 16 )	
BFslider:SetScript( "OnValueChanged", CmdListDisplayUpdate[Mode] )
getglobal(BFslider:GetName() .. 'Low'):SetText( " " )
getglobal(BFslider:GetName() .. 'High'):SetText( " " )
BFslider:SetScript("OnMouseWheel", function(self, delta)
	SliderMW(self, -delta)
end)
BFslider.HintText = "Scroll the list with your mousewheel. (Holding shift will scroll 5x faster)"
BFslider:SetScript("OnEnter", HintSet )
BFslider:SetScript("OnLeave", HintClear )

do
	local Button, ButtonName, lastButton
	for x = 1, 10 do
		ButtonName = "BottomButton"..x
		me[ButtonName] = NewButton( me, "RED", "SM"..ButtonName.."Button" )
		Button = me[ButtonName]
		if x == 1 then
			Button:SetPoint("BOTTOM", me, "BOTTOM", 0, 10)
			Button:SetPoint("LEFT", me.ButtonFrame, "LEFT")
		else
			Button:SetPoint("LEFT", lastButton, "RIGHT")
		end
		Button:SetSize( 150, 25)
		Button:SetText(ButtonName)
		--Button.Name = ButtonName
		Button:SetScript("OnEnter", BottomButtonHintSet )
		Button:SetScript("OnLeave", HintClear )
		Button:SetScript("OnClick", function(self, button, down)
			if button ~= "LeftButton" or self:GetSkin() == "GREY" then
				return
			end
			PlaySound( 624 ) -- "GAMEGENERICBUTTONPRESS"
			BottomButtonHandler[self.Name]()
			CmdListDisplayUpdate[Mode]()
		end)
		lastButton = Button
	end
end

me.NameFrame = CreateFrame("Frame" , "SMNameFrame", me, "BackdropTemplate" )
local NameFrame = me.NameFrame
NameFrame:SetBackdrop( Backdrop2 )
NameFrame:SetBackdropColor( 128, 128,128, 1)
NameFrame:SetBackdropBorderColor( 255, 255, 255, 1)
NameFrame:SetPoint( "TOPLEFT", me.ButtonFrame, "TOPRIGHT", 10, 0 )
NameFrame:SetHeight( 30 )
NameFrame:SetPoint( "RIGHT", me, "RIGHT", -10, 0)

me.NameFrameText = me.NameFrame:CreateFontString( nil , "OVERLAY", "ChatFontNormal")
local NameFrameText = me.NameFrameText
NameFrameText:SetPoint( "TOPLEFT", NameFrame, "TOPLEFT", 8, -8 )
NameFrameText:SetPoint( "BOTTOM", NameFrame, "BOTTOM", 0, 0 )
NameFrameText:SetWidth( 200 )
NameFrameText:SetJustifyH( "LEFT" )
NameFrameText:SetJustifyV( "TOP" )

me.AliasText = me.NameFrame:CreateFontString( nil , "OVERLAY", "ChatFontNormal")
local AliasText = me.AliasText
AliasText:SetPoint( "TOPLEFT", NameFrameText, "TOPRIGHT", 8, 0 )
AliasText:SetPoint( "BOTTOMRIGHT", NameFrameText, "BOTTOMRIGHT", 40, 0 )
AliasText:SetJustifyH( "LEFT" )
AliasText:SetJustifyV( "TOP" )
AliasText:SetText("Alias:")

me.VersionText =  NameFrame:CreateFontString( nil , "OVERLAY", "ChatFontNormal")
local VersionText = me.VersionText
VersionText:SetPoint( "LEFT", NameFrame, "RIGHT", -120, 0)
VersionText:SetPoint( "BOTTOM", NameFrameText, "BOTTOM")
VersionText:SetPoint( "TOP", NameFrameText, "TOP")
VersionText:SetPoint( "RIGHT", NameFrame, "RIGHT", -10, 0 )
VersionText:SetJustifyH( "RIGHT" )
VersionText:SetJustifyV( "TOP" )

me.AliasEBox = CreateFrame( "EditBox", "SMAliasEditbox", me.NameFrame, "BackdropTemplate" )
local EBox = me.AliasEBox
EBox:SetFontObject("ChatFontNormal")
EBox:SetTextInsets(5, 5, 3, 3)
EBox:SetPoint("TOPLEFT", me.AliasText, "TOPRIGHT", 0, 3 )
EBox:SetPoint("BOTTOM", me.AliasText, "BOTTOM", 0, 5 )
EBox:SetPoint("RIGHT", me.VersionText, "LEFT", -10, 0)
EBox:SetMultiLine(false)
EBox:SetBackdrop( Backdrop3 )
EBox:SetAutoFocus(false)
EBox:SetMaxLetters( 128 )
EBox:SetScript("OnEnterPressed" , function(self)
	self:ClearFocus()
end)
EBox:SetScript("OnEditFocusGained", function(self)
	self:HighlightText()
end)
EBox:SetScript("OnEscapePressed", function(self)
	self:SetCursorPosition( 0 )
	self:ClearFocus()
end)

EBox.HintText = "Set aliases to the command here with a space separated list (no slashes please)"
EBox:SetScript("OnEnter", HintSet )
EBox:SetScript("OnLeave", HintClear )

me.DescFrame = CreateFrame("Frame" , "SMDescFrame", me, "BackdropTemplate" )
me.DescFrame:SetBackdrop( Backdrop2 )
me.DescFrame:SetBackdropColor( 128, 128,128, 1)
me.DescFrame:SetBackdropBorderColor( 255, 255, 255, 1)
me.DescFrame:SetPoint( "TOPLEFT", me.NameFrame, "BOTTOMLEFT", 0, -10 )
me.DescFrame:SetHeight( 60 )
me.DescFrame:SetPoint( "RIGHT", me.NameFrame, "RIGHT" )

me.DescFrameText = me.DescFrame:CreateFontString( nil , "OVERLAY", "ChatFontNormal")
me.DescFrameText:SetPoint( "TOPLEFT", me.DescFrame, "TOPLEFT", 8, -8 )
me.DescFrameText:SetPoint( "BOTTOM", me.DescFrame, "BOTTOM" )
me.DescFrameText:SetWidth( 40 )
me.DescFrameText:SetJustifyH( "LEFT" )
me.DescFrameText:SetJustifyV( "TOP" )
me.DescFrameText:SetText("Desc:")

me.DescFrameEBox = CreateFrame( "EditBox", "SMDescFrameEditbox", me.DescFrame, "BackdropTemplate" )
local EBox = me.DescFrameEBox
EBox:SetFontObject("ChatFontNormal")
EBox:SetTextInsets(5, 5, 3, 3)
EBox:SetPoint("TOPLEFT", me.DescFrameText, "TOPRIGHT", 0 , 3 )
EBox:SetPoint("BOTTOMRIGHT", me.DescFrame, "BOTTOM", 5, 5 )
EBox:SetPoint("RIGHT", me.DescFrame, "RIGHT", -5, 0)
EBox:SetMultiLine(true)
EBox:SetBackdrop(Backdrop3)
EBox:SetAutoFocus(false)
EBox:SetMaxLetters( 512 )
EBox:SetScript("OnEditFocusGained", function(self)
	self:HighlightText()
end)
EBox:SetScript("OnEscapePressed", function(self)
	self:SetCursorPosition( 0 )
	self:ClearFocus()
end)

EBox.HintText = "Set the description of your command here"
EBox:SetScript("OnEnter", HintSet )
EBox:SetScript("OnLeave", HintClear )

me.CodeFrame = CreateFrame("Frame" , "SMCodeFrame", me, "BackdropTemplate" )
me.CodeFrame:SetBackdrop( Backdrop2 )
me.CodeFrame:SetBackdropColor( 128, 128,128, 1)
me.CodeFrame:SetBackdropBorderColor( 255, 255, 255, 1)
me.CodeFrame:SetPoint( "TOPLEFT", me.DescFrame, "BOTTOMLEFT", 0, -10 )
me.CodeFrame:SetPoint( "BOTTOM", me.ButtonFrame, "BOTTOM" )
me.CodeFrame:SetPoint( "RIGHT", me.DescFrame, "RIGHT") 

me.CodeFrame.HintText = "Edit the code for your slash command here. Your function will be called as: function(msg)     For help with text colors and addon provided functions click the help tab above. When switching between tabs your code changes may be lost if you do not save the changes first."
me.CodeFrame:SetScript("OnEnter", HintSet )
me.CodeFrame:SetScript("OnLeave", HintClear )

me.CodeFrameText = me.CodeFrame:CreateFontString( nil , "OVERLAY", "ChatFontNormal")
me.CodeFrameText:SetPoint( "TOPLEFT", me.CodeFrame, "TOPLEFT", 8, -8 )
me.CodeFrameText:SetPoint( "BOTTOM", me.CodeFrame, "BOTTOM" )
me.CodeFrameText:SetWidth( 40 )
me.CodeFrameText:SetJustifyH( "LEFT" )
me.CodeFrameText:SetJustifyV( "TOP" )
me.CodeFrameText:SetText("Code:")

me.EditCodeFrame = CreateFrame("Frame" , "SMEditCodeFrame", me.CodeFrame )
local me2 = me.EditCodeFrame
me2:SetWidth( me.CodeFrame:GetWidth() )
me2:SetHeight( me.CodeFrame:GetHeight() )
me2.HintText = me.CodeFrame.HintText
me2:SetScript("OnEnter", HintSet )
me2:SetScript("OnLeave", HintClear )

-- Scroll Frame
me.ScrollFrame = CreateFrame("ScrollFrame", "SMScrollFrame", me.CodeFrame, "UIPanelScrollFrameTemplate")
me.ScrollFrame:SetScrollChild(me2)
me.ScrollFrame:SetPoint("TOPLEFT", me.CodeFrameText, "TOPRIGHT", 0 , 3 )
me.ScrollFrame:SetPoint("BOTTOM", me.CodeFrame, "BOTTOM", 0, 5 )
me.ScrollFrame:SetPoint("RIGHT", me.CodeFrame, "RIGHT", -5, 0)

me.LineNumbers = me2:CreateFontString( nil, "OVERLAY")
me.LineNumbers:SetFontObject("ChatFontNormal")
me.LineNumbers:SetFont("Interface\\addons\\SlashMagicConfig\\VeraMono.ttf", 12 )
me.LineNumbers:SetPoint("TOPLEFT", me2, "TOPLEFT",0 ,-8)
me.LineNumbers:SetJustifyH("RIGHT")
me.LineNumbers:SetWidth(35)
me.LineNumbers:SetText("1:\r")

me.CodeFrameEBox = CreateFrame( "EditBox", "SMCodeFrameEditbox", me2)
local EBox = me.CodeFrameEBox

EBox:SetFontObject("ChatFontNormal")
EBox:SetTextInsets(5, 5, 3, 3)
EBox:SetPoint("TOPLEFT", me.LineNumbers, "TOPRIGHT", 0 , 3 )
EBox:SetMultiLine(true)
EBox:SetFont("Interface\\addons\\SlashMagicConfig\\VeraMono.ttf", 12 )
EBox:SetAutoFocus(false)
EBox:SetMaxLetters( 8192 )
EBox:SetScript("OnEditFocusGained", function(self)
	self:HighlightText()
end)
EBox:SetScript("OnEscapePressed", function(self)
	self:SetCursorPosition( 0 )
	self:ClearFocus()
end)
EBox:SetScript("OnTabPressed", function(self)
	if not( self.TabWaiting ) then self.TabWaiting = 0 end
	self.TabWaiting = TabWaiting + 1
end)

EBox.HintText = me.CodeFrame.HintText
EBox:SetScript("OnEnter", HintSet )
EBox:SetScript("OnLeave", HintClear )

-- Line Number measurement fontstring
me.StrLen = me:CreateFontString(nil, "BACKGROUND" )
local StrLen = me.StrLen
StrLen:SetFontObject("ChatFontNormal")
StrLen:SetPoint("TOPLEFT", EBox, "TOPLEFT", 5, 0)
StrLen:SetPoint("RIGHT", EBox, "RIGHT", -5, 0)
StrLen:SetFont("Interface\\addons\\SlashMagicConfig\\VeraMono.ttf", 12 )
StrLen:SetJustifyH("LEFT")
StrLen:Hide()
StrLen:SetNonSpaceWrap()

IndentationLib.Enable(EBox, 5, ColorTable)

me.HintFrame = CreateFrame("Frame" , "SMHintFrame", me, "BackdropTemplate" )
me.HintFrame:SetBackdrop( Backdrop2 )
me.HintFrame:SetBackdropColor( 128, 128,128, 1)
me.HintFrame:SetBackdropBorderColor( 255, 255, 255, 1)
me.HintFrame:SetPoint( "TOPLEFT", me.ButtonFrame, "BOTTOMLEFT", 0, -10 )
me.HintFrame:SetPoint( "BOTTOM", me, "BOTTOM", 0, 35 )
me.HintFrame:SetPoint( "RIGHT", me, "RIGHT", -10, 0) 

me.HintFrameText = me:CreateFontString( nil , "OVERLAY", "ChatFontNormal")
me.HintFrameText:SetAllPoints( me.HintFrame )
me.HintFrameText:SetJustifyH( "CENTER" )
me.HintFrameText:SetJustifyV( "MIDDLE" )
me.HintFrameText:SetText(" ")

me.pos = me.CodeFrame:CreateFontString( nil , "OVERLAY", "ChatFontNormal")
me.pos:SetPoint("TOPRIGHT", me.HintFrame, "TOPRIGHT", -5, -5 )
me.pos:SetJustifyH( "RIGHT" )
me.pos:SetJustifyV( "TOP" )
me.pos:SetText(" ")

-- Code EBox Overlay
me.EBoxOverlay=CreateFrame("Frame", "SMOverlayFrame" ,me.CodeFrame, "BackdropTemplate")
me.EBoxOverlay:SetBackdrop(Backdrop3)

me.EBoxOverlay.HintText = me.CodeFrame.HintText
me.EBoxOverlay:SetScript("OnEnter", HintSet )
me.EBoxOverlay:SetScript("OnLeave", HintClear )
me.EBoxOverlay:SetPoint("TOP", me.CodeFrame, "TOP", 0, -6)
me.EBoxOverlay:SetPoint("LEFT", me.LineNumbers, "RIGHT", 0, 0)
me.EBoxOverlay:SetPoint("RIGHT", me.DescFrameEBox, "RIGHT", 0, 0)
me.EBoxOverlay:SetPoint("BOTTOM", me.CodeFrame, "BOTTOM", 0, 6)

-- ****************************************************************************
-- * Settings Sub Frame
me.SettingsFrame = CreateFrame("Frame" , "SMSettingsFrame", me, "BackdropTemplate" )
local Settings = me.SettingsFrame
Settings:SetBackdrop( Backdrop2 )
Settings:SetBackdropColor( 128, 128,128, 1)
Settings:SetBackdropBorderColor( 255, 255, 255, 1)
Settings:SetPoint( "TOPLEFT", me.DescFrame, "BOTTOMLEFT", 0, -10 )
Settings:SetPoint( "BOTTOM", me.ButtonFrame, "BOTTOM" )
Settings:SetPoint( "RIGHT", me.DescFrame, "RIGHT") 

local SettingButtonHints = {
	["SETTINGS_EnableAddon"] = "This will enable/disable the addon. Slash commands will not work when the addon is disabled but you can edit them. This setting takes effect immediately.",
	["SETTINGS_ResetAddon"] = "This button will RESET the addon back to the initial install state",
	["SETTINGS_LoadDefaultCommands"] = "This button toggles the loading of the built in default commands",
	
}

do
	me.EnableAddon = ChkBox( nil, "SMConfig", "EnableAddon", Settings, CheckClick, "Enables the addon to function", nil, 15, -30 )
	local Enable = me.EnableAddon
	Enable.HintText = "This will enable/disable the addon. Slash commands will not work when the addon is disabled but you can edit them. This setting takes effect immediately."
	Enable:SetScript("OnEnter", HintSet )
	Enable:SetScript("OnLeave", HintClear )
	Enable:SetScript("OnClick", function(self)
		SMS.Enable = self:GetChecked()
		if SMS.Enable then
			SM.AddonEnable()
		else
			SM.AddonDisable()
		end
	end)

	me.LoadDefaultCommands = ChkBox( Enable, "SMConfig", "LoadDefaultCommands", Settings, CheckClick, "Enables the built in default commands" )
	local LDC = me.LoadDefaultCommands
	LDC.HintText = "This will enable/disable the loading of the default commands. This will require a reload of the UI to take effect."
	LDC:SetScript("OnEnter", HintSet )
	LDC:SetScript("OnLeave", HintClear )
	LDC:SetScript("OnClick", function(self)
		local dialog
		if SMS.LoadDefaultCommands then
			dialog = StaticPopup_Show("SLASHMAGIC", "Are you sure you want to disable all default commands? (This action will require a reload of the UI to unload the default commands)" )
		else
			dialog = StaticPopup_Show("SLASHMAGIC", "Are you sure you want to enable loading of the default commands? (This action will require a reload of the UI to enable the default commands)" )
		end
		dialog.data = "LOAD DEFAULT COMMANDS"
		dialog.button1:SetText("OK")
		dialog.button1:Enable()
	end)

	me.ShowGems = ChkBox( LDC, "SMConfig", "ShowGems", Settings, CheckClick, "Enables showing the gems on built in default commands" )
	local Gems = me.ShowGems
	Gems.HintText = "This will enable/disable the gems marking default commands."
	Gems:SetScript("OnEnter", HintSet )
	Gems:SetScript("OnLeave", HintClear )
	Gems:SetScript("OnClick", function(self)
		SMS.ShowGems = self:GetChecked()
	end)

	me.ShowDead = ChkBox( Gems, "SMConfig", "ShowDead", Settings, CheckClick, "Enables showing dead built in default commands" )
	local Dead = me.ShowDead
	Dead.HintText = "This will enable/disable the commands that are now dead as Blizzard changed/removed/ or protected their function."
	Dead:SetScript("OnEnter", HintSet )
	Dead:SetScript("OnLeave", HintClear )
	Dead:SetScript("OnClick", function(self)
		SMS.ShowDead = self:GetChecked()
	end)

	me.EnableAutoRuns = ChkBox( Dead, "SMConfig", "EnableAutoRuns", Settings, CheckClick, "Enables the running of saved commands at startup")
	local EAR = me.EnableAutoRuns
	EAR.HintText = "This will enable/disable the addon from running a saved list of commands once the addon loads. Not all commands can be ran this way"
	EAR:SetScript("OnEnter", HintSet )
	EAR:SetScript("OnLeave", HintClear )
	EAR:SetScript("OnClick", function(self)
		SMS.EnableAutoRuns = self:GetChecked()
	end)

	me.EnableArchives = ChkBox( EAR, "SMConfig", "EnableArchives", Settings, CheckClick, "Enables the archiving of default commands incase they disappear from the addon")
	local EArc = me.EnableArchives
	EArc.HintText = "This will enable/disable the addon from archiving default commands incase they are no longer included in the addon"
	EArc:SetScript("OnEnter", HintSet )
	EArc:SetScript("OnLeave", HintClear )
	EArc:SetScript("OnClick", function(self)
		SMS.EnableArchives = self:GetChecked()
	end)
	
	me.DeleteArchives = CreateFrame( "Button", "SMDeleteArchivesButton", Settings, "UIPanelButtonTemplate" )
	local DelArc = me.DeleteArchives
	DelArc.HintText = ("This button will delete all saved command archives")
	DelArc:SetScript("OnEnter", HintSet )
	DelArc:SetScript("OnLeave", HintClear )
	DelArc:SetPoint("TOPLEFT", EArc, "BOTTOMLEFT", 0, -10)
	DelArc:SetSize( 150, 25)
	DelArc:SetText("Delete Archives")
	DelArc:SetScript("OnClick", function(self, button, down)
		if button ~= "LeftButton" then
			return
		end
		PlaySound( 624 ) -- "GAMEGENERICBUTTONPRESS"
		
	end)
	
	me.ResetAddon = CreateFrame( "Button", "SMResetAddonButton", Settings, "UIPanelButtonTemplate" )
	local RA = me.ResetAddon
	RA.HintText = ("This button will reset the addon to \"Factory\" install")
	RA:SetScript("OnEnter", HintSet )
	RA:SetScript("OnLeave", HintClear )
	RA:SetPoint("TOPLEFT", DelArc, "BOTTOMLEFT", 0, -10)
	RA:SetSize( 150, 25)
	RA:SetText("Reset Addon")
	RA:SetScript("OnClick", function(self, button, down)
		if button ~= "LeftButton" then
			return
		end
		PlaySound( 624 ) -- "GAMEGENERICBUTTONPRESS"
		
		local dialog = StaticPopup_Show("SLASHMAGIC", "Are you sure you want to reset the addon back to \"Factory\" conditions. This will remove all custom created commands and restore the original functionality to the addon." )
		dialog.data = "RESET ADDON"
		dialog.button1:SetText("Reset")
		dialog.button1:Enable()
	end)

end
--[[do
	local ButtonList = { "SETTINGS_ResetAddon", "SETTINGS_LoadDefaultCommands",}
	local Button, ButtonName, lastButton
	for x = 1, #ButtonList do
		ButtonName = ButtonList[x]
		Settings[ButtonName] = NewButton( Settings, "RED", "SM"..ButtonName.."Button" )
		Button = me[ButtonName]
		if x == 1 then
			Button:SetPoint("TOPLEFT", Settings, "TOPLEFT", 10, -10)
		else
			Button:SetPoint("TOPLEFT", lastButton, "BOTTOMLEFT")
		end
		Button:SetSize( 150, 25)
		Button:SetText(ButtonName)
		Button.Name = ButtonName
		Button.HintText = SettingButtonHints[ButtonName]
		Button:SetScript("OnEnter", HintSet )
		Button:SetScript("OnLeave", HintClear )
		Button:SetScript("OnClick", function(self, button, down)
			if button ~= "LeftButton" or self:GetSkin() == "GREY" then
				return
			end
			PlaySound( 624 ) -- "GAMEGENERICBUTTONPRESS"
			SettingButtonHandler[self.Name]()
			CmdListDisplayUpdate[Mode]()
		end)
		lastButton = Button
	end
end
]]

-- ****************************************************************************
-- * Line Numbers/Columns and EditBox Scroll Positioning                      *
-- ****************************************************************************

local eframe = 0
local e = 0
local charHeight
me2:SetScript("OnUpdate", function(self, elapsed)
	if (Mode == Settings) then return end
	eframe = eframe + 1
	e = e + elapsed
	if eframe < 2 and not(me.CodeFrameEBox.TabWaiting) then return end
	eframe = 0
	
	local cursorPos = me.CodeFrameEBox:GetCursorPosition()
	if oldPos ~= cursorPos or e > .5 or me.CodeFrameEBox.TabWaiting then
		e = 0
		pos = cursorPos
		if not(charHeight) then
			StrLen:SetText("X")
			charHeight = StrLen:GetStringHeight()
		end
		
		local CRs
		local code = { string.split("\r\n", me.CodeFrameEBox:GetText()) }
		local linestring = ""
		local line = false
		local currentLinePixels = 0
		
		-- Line Numbers and column section
		for x = 1, #code do
			code[x] = ichars( code[x], true)
			
			StrLen:SetText( code[x]:len() > 0 and code[x] or "X")
			
			CRs = math.floor(StrLen:GetStringHeight() / charHeight)
			linestring = linestring..x..":"..string.rep("\r", CRs)
			
			if not(line) and pos - code[x]:len() <= 0 then
				line = x
				pos = (code[x] and code[x]:sub(1,pos) or ""):len()
				me.pos:SetText( ("line: %d  char: %d"):format(line, pos))
			elseif not( line ) then
				currentLinePixels = currentLinePixels + (CRs * charHeight)
				pos = (pos - 1) - (code[x] and code[x]:len() or 0)
			end
		end
		
		if me.CodeFrameEBox.TabWaiting then
			local tab = (pos % 5 > 0) and (pos % 5) or 5
			me.CodeFrameEBox:Insert(string.rep(" ", tab))
			me.CodeFrameEBox:SetCursorPosition(cursorPos + tab)
			me.CodeFrameEBox.TabWaiting = me.CodeFrameEBox.TabWaiting - 1
		end

		linestring = linestring:gsub( ("%d:"):format(line), ("|cFF00FF00%d:|r"):format(line), 1)
		me.LineNumbers:SetText(linestring.."\n\n\nEOF\r")
		oldPos = cursorPos
		
		-- scroll frame adjust section
		if oldLinePixels ~= currentLinePixels and not( IsMouseButtonDown("LeftButton") ) then
			local displayPixels = me.EBoxOverlay:GetHeight() --
			local displayTop = me.ScrollFrame:GetVerticalScroll()
			local displayBottom = displayTop + displayPixels

			if currentLinePixels > displayTop + EDGE and currentLinePixels < displayBottom - EDGE then
				-- within viewing window so do nothing
			elseif currentLinePixels < displayTop + EDGE and displayTop > 1 and currentLinePixels < oldLinePixels then
				me.ScrollFrame:SetVerticalScroll( currentLinePixels - EDGE )
			elseif currentLinePixels > displayBottom - EDGE then
				me.ScrollFrame:SetVerticalScroll( currentLinePixels - displayPixels + EDGE)
			end
			oldLinePixels = currentLinePixels
		end
	end
	
end)

-- ****************************************************************************
-- * New Command Entry Frame                                                  *
-- ****************************************************************************

local newCommandAcceptDialogHandler = function(self, data, data2)
	local NewCmd = string.lower(self.editBox:GetText())
	local slashCommand = "/"..strupper(NewCmd)
	local msg
	
	if SlashMagic_Commands[NewCmd] then
		msg = ("The command |cFFFF0000[ %s ]|r already exists as part of this addon. Please edit that command or choose a different name."):format(NewCmd)
	elseif _G["hash_SlashCmdList"][slashCommand] or _G["hash_ChatTypeInfoList"][slashCommand] then
		msg = string.join("", color.Yellow, "  Error: ", color.Red, NewCmd, color.Yellow, " is an existing slash command not provided by this addon. Please choose a different name.", color.Reset)
	elseif _G["hash_EmoteTokenList"][slashCommand] then
		msg = string.join("",  color.Yellow, "  Error: ", color.Red, NewCmd, color.Yellow, " is an existing emote command not provided by this addon. Please choose a different name.", color.Reset)
	end
	
	if msg then
		self.text:SetText( msg )
		self.editBox:SetText("")
		self.editBox:SetFocus()
		return true
	end
	
	SlashMagic_Commands[NewCmd] = {}
	local command = SlashMagic_Commands[NewCmd]
	command.desc = "Enter a description here"
	command.vers = "Custom"
	command.alias = ""
	command.state = false
	command.code = { "-- Put some code here and make your own command",
					 "return" ,}
	Selected = NewCmd
	CmdListDisplayUpdate[Mode]()
end

StaticPopupDialogs["SLASHMAGIC_NEWCOMMAND"] = {
	text = "Enter the name of your new command (no slashes needed)",
	button1 = "Create",
	button2 = "Cancel",
	OnAccept = newCommandAcceptDialogHandler,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	hasEditBox = true,
	OnShow = function(self, data)
		self.editBox:SetScript("OnEnterPressed", function(self)
			self:GetParent().button1:Click("LeftButton")
		end)
		self.editBox:SetScript("OnEscapePressed", function(self)
			self:GetParent().button2:Click("LeftButton")
		end)
	end,
	preferredIndex = 3,  -- avoid some UI taint, see http://www.wowace.com/announcements/how-to-avoid-some-ui-taint/ 
}
